// ==============================================================
//                 ORBITER MODULE: CM_Apex
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Apex.cpp
// Control module for CM_Apex vessel class
//
// Notes:
// ==============================================================

#include "CM_Apex.h"

// ==============================================================
// Class Constructor and Destructor
// ==============================================================

CM_Apex::CM_Apex (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
	birthdate = oapiGetSimTime ();	// Birthdate = current sim time
	lifespan = 3600;				// Set default lifespan, 3600 seconds (1 hour).
}

CM_Apex::~CM_Apex ()
{
}

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void CM_Apex::clbkSetClassCaps (FILEHANDLE cfg)
{
	// physical vessel parameters
	SetSize (APEX_SIZE);
	SetEmptyMass (APEX_MASS);
	SetPMI (APEX_PMI);
	SetCrossSections (APEX_CS);
	SetRotDrag (APEX_RD);
	SetCW (APEX_CW[0], APEX_CW[1], APEX_CW[2], APEX_CW[3]);

	SetDockParams (APEX_DOCKPOS, _V( 0, 0, 1), _V(0, 1, 0));
	CreateAttachment (true, APEX_DOCKPOS, _V( 0, 0, 1), _V( 0, 1, 0), "X");

	// associate a mesh for the visual
	MESHHANDLE mh_apex = oapiLoadMeshGlobal ("AAPO/CM_Apex");
	AddMesh (mh_apex);
}

// --------------------------------------------------------------
// Post step processes
// --------------------------------------------------------------
void CM_Apex::clbkPostStep (double simt, double simdt, double mjd)
{
	// Clean up
	OBJHANDLE self = GetHandle ();					// Get vessel handle 

	if (GroundContact ()) oapiDeleteVessel(self);	// If vessel touches the ground delete it 

	age = oapiGetSimTime () - birthdate;			// Get age, current sim time - birthdate
	if (age > lifespan) oapiDeleteVessel(self);		// If age is greater than lifespan delete vessel 

	double drag = GetDrag () / GetMass ();			// Get decceleration due to drag
	if (drag > 8 * G) oapiDeleteVessel (self);		// if decceleration is greater than 8 Gs burn up
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new CM_Apex (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (CM_Apex*)vessel;
}
