// ==============================================================
//                 ORBITER MODULE: CM_Apex
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Apex.cpp
// Control module for CM_Apex vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#pragma once
#include "orbitersdk.h"
#include "AAPO_Common.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double	APEX_SIZE		= 0.85;							// Vessel's mean radius [m]
const VECTOR3	APEX_CS			= { 1.02, 1.02, 2.50};			// x,y,z cross sections [m^2]
const VECTOR3	APEX_PMI		= { 2.3, 2.3, 0.8};				// Principal moments of inertia (mass-normalised) [m^2]
const VECTOR3	APEX_RD			= { 0.025, 0.025, 0.02};		// Rotational drag coefficients
const VECTOR3	APEX_COP		= { 0.0, 0.0,-0.3};				// Center of mass
const double	APEX_CW[4]		= { 0.2, 1.0, 0.47, 0.47};		// Wind resistance coefficients (forward, aft, lateral, vertical)
const double	APEX_MASS		= 100;							// Apex (parachute cover) mass [kg]

const VECTOR3	APEX_DOCKPOS	= { 0.0, 0.0, 0.43};			// Docking port

// ==============================================================
// Vessel class interface
// ==============================================================

class CM_Apex: public VESSEL3 {
public:
	CM_Apex (OBJHANDLE hVessel, int flightmodel);
	~CM_Apex ();

	void	clbkSetClassCaps (FILEHANDLE cfg);						// Set vessel class caps
	void	clbkPostStep (double simt, double simdt, double mjd);	// Post-Step processes

private:
	double birthdate, age, lifespan;
};