// ==============================================================
//                 ORBITER MODULE: CM_Chute
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Chute.cpp
// Control module for CM_Chute vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#pragma once
#include "orbitersdk.h"
#include "AAPO_Common.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double	CHUTE_SIZE			= 85.5 * METERS;			// Vessel's mean radius [m]
const VECTOR3	CHUTE_CS			= { 190, 205, 260};			// x,y,z cross sections [m^2]
const VECTOR3	CHUTE_PMI			= { 2.3, 2.3, 0.8};			// Principal moments of inertia (mass-normalised) [m^2]
const VECTOR3	CHUTE_RD			= { 0.4, 0.4, 0.1};			// Rotational drag coefficients
const VECTOR3	CHUTE_COM			= { 0.0, 0.0, 58};			// Center of mass
const double	CHUTE_CW[4]			= { 0.47, 1.5, 0.47, 0.47};	// Wind resistance coefficients (forward, aft, lateral, vertical)
const double	CHUTE_MASS			= 127 * KILOS;				// Main parachute mass [kg] (127 lbs each, Apollo CSM News Reference pg 95)
const double	CHUTE_AREA			= PI * pow (12.725, 2);		// Main parachute cross section [m^2] ("PI * R squared" applied to diameter of 83.5 ft, Apollo CSM News Reference pg 95)
const double	CHUTE_LENGTH		= 30;						// Main parachute drogue height above (120 ft, Apollo CSM News Reference pg 95)

const double	CHUTE_DTIME			= 10;						// Parachute deployment time [s]
const int		CHUTE_NGROUPS		= 3;

// ==============================================================
// Vessel class interface
// ==============================================================

class CM_Chute: public VESSEL3 {
public:
	CM_Chute (OBJHANDLE hVessel, int flightmodel);
	~CM_Chute ();

	void	clbkSetClassCaps (FILEHANDLE cfg);						// Set vessel class caps
	void	clbkLoadStateEx (FILEHANDLE scn, void *status);			// Read status from scenario file
	void	clbkSaveState (FILEHANDLE scn);							// Write status to scenario file
	void	clbkPostCreation ();									// Finalise vessel creation
	void	clbkPostStep (double simt, double simdt, double mjd);	// Post-Step processes
	
private:
	double birthdate, age, lifespan;
	enum doorstate	ChuteStatus;

	// Animations
	UINT		anim_deploy, anim_deflate;
	double		deploy_proc, deflate_proc;
};
