// ==============================================================
//                 ORBITER MODULE: CM_Drogue
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Drogue.cpp
// Control module for CM_Drogue vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#pragma once
#include "orbitersdk.h"
#include "AAPO_Common.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double	DROGUE_SIZE			= 16.5 * METERS;			// Vessel's mean radius [m]
const VECTOR3	DROGUE_CS			= { 12, 18, 30};			// x,y,z cross sections [m^2]
const VECTOR3	DROGUE_PMI			= { 2.3, 2.3, 0.8};			// Principal moments of inertia (mass-normalised) [m^2]
const VECTOR3	DROGUE_RD			= { 0.4, 0.4, 0.1};			// Rotational drag coefficients
const VECTOR3	DROGUE_COM			= { 0.0, 0.0, 58};			// Center of mass
const double	DROGUE_CW[4]		= { 0.47, 1.5, 0.47, 0.47};	// Wind resistance coefficients (forward, aft, lateral, vertical)
const double	DROGUE_MASS			= 25.09 * 2;				// Stabilization drogue mass [kg] (Estimated using main parachute mass)
const double	DROGUE_AREA			= PI * pow (2.5, 2);		// Stabilization drogue cross section [m^2] ("PI * R squared" applied to diameter of 16.5 ft, Apollo CSM News Reference pg 94)
const double	DROGUE_LENGTH		= 65 * METERS;				// Stabilization drogue height above CM (65 ft, Apollo CSM News Reference pg 94)

const double	DROGUE_DTIME		= 3.0;						// Parachute deployment time [s]
const int		DROGUE_NGROUPS		= 3;

// ==============================================================
// Vessel class interface
// ==============================================================

class CM_Drogue: public VESSEL3 {
public:
	CM_Drogue (OBJHANDLE hVessel, int flightmodel);
	~CM_Drogue ();

	void	clbkSetClassCaps (FILEHANDLE cfg);						// Set vessel class caps
	void	clbkLoadStateEx (FILEHANDLE scn, void *status);			// Read status from scenario file
	void	clbkSaveState (FILEHANDLE scn);							// Write status to scenario file
	void	clbkPostCreation ();									// Finalise vessel creation
	void	clbkPostStep (double simt, double simdt, double mjd);	// Post-Step processes
	
private:
	double birthdate, age, lifespan;
	enum doorstate	ChuteStatus;

	// Animations
	UINT		anim_deploy;
	double		deploy_proc;
};
