// ==============================================================
//                 ORBITER MODULE: CM_Probe
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Probe.cpp
// Control module for CM_Probe vessel class
//
// Notes:
// ==============================================================

#include "CM_Probe.h"

// ==============================================================
// Class Constructor and Destructor
// ==============================================================

CM_Probe::CM_Probe (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
	birthdate = oapiGetSimTime ();	// Birthdate = current sim time
	lifespan = 3600;				// Set default lifespan, 3600 seconds (1 hour).
}

CM_Probe::~CM_Probe ()
{
}

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set vessel class capabilities 
// --------------------------------------------------------------
void CM_Probe::clbkSetClassCaps (FILEHANDLE cfg)
{
	// physical vessel parameters
	SetSize (PROBE_SIZE);
	SetEmptyMass (PROBE_MASS);
	SetPMI (PROBE_PMI);
	SetCrossSections (PROBE_CS);
	SetRotDrag (PROBE_RD);
	SetCW (PROBE_CW[0], PROBE_CW[1], PROBE_CW[2], PROBE_CW[3]);

	SetDockParams (_V( 0, 0, 0), _V( 0, 0, 1), _V( 0, 1, 0));
	CreateAttachment (true, _V( 0, 0, 0), _V( 0, 0,-1), _V( 0, 1, 0), "X");
	CreateAttachment (false, _V( 0, 0, 0), _V( 0, 0,-1), _V( 0, 1, 0), "X");

	// associate a mesh for the visual
	MESHHANDLE mh_probe = oapiLoadMeshGlobal ("AAPO/DockingProbe");
	AddMesh (mh_probe);
}

// --------------------------------------------------------------
// Post step processes
// --------------------------------------------------------------
void CM_Probe::clbkPostStep (double simt, double simdt, double mjd)
{
	// Clean up
	OBJHANDLE self = GetHandle ();					// Get vessel handle 

	if (GroundContact ()) oapiDeleteVessel(self);	// If vessel touches the ground delete it 

	age = oapiGetSimTime () - birthdate;			// Get age, current sim time - birthdate
	if (age > lifespan) oapiDeleteVessel(self);		// If age is greater than lifespan delete vessel 

	double drag = GetDrag () / GetMass ();			// Get decceleration due to drag
	if (drag > 8 * G) oapiDeleteVessel (self);		// if decceleration is greater than 8 Gs burn up
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new CM_Probe (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (CM_Probe*)vessel;
}
