// ==============================================================
//                    ORBITER MODULE: LM
//                  Part of the ORBITER SDK
//               Copyright (C) 2012 Greg Hlynka
//                    All rights reserved
//
// LM_Cockpit.h
// Interface for LM vessel class
//
// Notes: This file exists
// ==============================================================

#pragma once
#include "orbitersdk.h"
#include "CSM.h"

// ==============================================================
// Definitions
// ==============================================================

// ==============================================================
// Constants
// ==============================================================
const int		CSM_VALVECOUNT		= 38;					// Propellant valve count

// ==============================================================
// Class declaration
// ==============================================================
class CSM_SYS 
{
friend class CSM;
friend class CSM_VC;

public:
	CSM_SYS (CSM *vessel, CSM_VC *cockpit);
	~CSM_SYS ();

	inline const CSM *GetVessel() const { return v; }

	// Public Functions
	void	InitCSM (void);										// Load initial state
	bool	ParseScenarioLine (char *line);						// Read status from scenario file
	void	SaveState (FILEHANDLE scn);							// Write status to scenario file
	void	PreStep (double simt, double simdt, double mjd);	// Pre-Step processes
	void	PostStep (double simt, double simdt, double mjd);	// Post-Step processes

private:
	CSM		*v;		// vessel interface
	CSM_VC	*vc;	// virtual cockpit interface	

	// Propellant Management
	void		PropellantPreStep (double simt, double simdt, double mjd);		// Pre-Step processes
	void		PropellantPostStep (double simt, double simdt, double mjd);		// Post-Step processes

	doorstate	PropValve[CSM_VALVECOUNT];
	double		valve_proc[CSM_VALVECOUNT];

	// Propulsion / Attitude Control
	void	ThrusterPreStep (double simt, double simdt, double mjd);		// Pre-Step processes
	void	ThrusterPostStep (double simt, double simdt, double mjd);		// Post-Step processes

	void	GimbalSPS (double simdt, double x, double y);					
	bool	PitchGimbalEnabled (void), YawGimbalEnabled (void);				// Is SPS gimbaling enabled
	UINT	anim_gimbalx, anim_gimbaly;										// SPS gimbal animations
	double	gimbalx_proc, gimbaly_proc;										// SPS gimbal process values		
	double	pitchinput, rollinput, yawinput;


	// Earth Landing System
	void	ELS_TimeStep (double simt, double simdt, double mjd);

	int		ELS_state;			
	double	ELS_count;			
	bool	ELS_SysArmed ();	// Is the system armed?
	bool	ELS_InAuto ();		// Is it in automatic or manual mode?
	double	BarAlt ();			// Calculate barometric altitude

	// Enviromental Control and Life Support
	void	EnviromentPreStep (double simt, double simdt, double mjd);		// Pre-Step processes
	void	EnviromentPostStep (double simt, double simdt, double mjd);		// Post-Step processes

	// Power Management

	// Communications and Navigation
	UINT						anim_HGApitch, anim_HGAyaw;
	ANIMATIONCOMPONENT_HANDLE	ach_HGApitch, ach_HGAyaw;

	// Misc variables and functions
	bool	AbortInProgress;
};