// ==============================================================
//                 Orbiter Module: LM_Cockpit
//    part of the Apollo Aplications Prject for Orbiter (AAPO)
//               copyright (c) 2012 greg hlynka
//                    all rights reserved
//
// lm_cockpit.cpp
// control module for Lunar Module virtual cockpit and displays
//
// notes:
// ==============================================================

#include "orbitersdk.h"
#include "AAPO_Common.h"
#include "LM_Cockpit.h"
#include "LM_Systems.h"

// ==============================================================
// LM Cockpit class interface
// ==============================================================

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
LM_COCKPIT::LM_COCKPIT (LM *vessel)
{
	v = vessel;
	int i = 0;

	// Clear caution and warning flags
	MasterCautionFlag = false;
	for (i = 0; i < 40; i++) CWA_Flag[i] = false;
	for (i = 0; i < LM_VC_NEEDLECOUNT; i++) gauge_proc[i] = 0;

	// Default switch positions when spawned by scenario editor.
	int P1_SwitchInit[P1_SWITCHCOUNT]					= { 0, 0, 2, 1, 2, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 1, 1, 0, 1, 0};	
	for (i = 0; i < P1_SWITCHCOUNT; i++) P1_Switch[i]	= (switchstate)P1_SwitchInit[i];

	int P1_DialInit[P1_DIALCOUNT]						= { 0};
	for (i = 0; i < P1_DIALCOUNT; i++) P1_Dial[i]		= (switchstate)P1_DialInit[i];

	// Panel 2
	int P2_SwitchInit[P2_SWITCHCOUNT]					= { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2};
	for (i = 0; i < P2_SWITCHCOUNT; i++) P2_Switch[i]	= (switchstate)P2_SwitchInit[i];

	int P2_DialInit[P2_DIALCOUNT]						= { 0, 0, 0, 0};
	for (i = 0; i < P2_DIALCOUNT; i++) P2_Dial[i]		= (switchstate)P2_DialInit[i];

	// Panel 3
	int P3_SwitchInit[P3_SWITCHCOUNT]					= { 2, 2, 1, 1, 0, 2, 2, 1, 2, 1, 2, 2, 1, 2, 1, 1, 1, 1, 1, 1, 0, 1, 0};
	for (i = 0; i < P3_SWITCHCOUNT; i++) P3_Switch[i]	= (switchstate)P3_SwitchInit[i];

	int P3_DialInit[P3_DIALCOUNT]						= { 0, 0, 0, 0, 0, 0};
	for (i = 0; i < P3_DIALCOUNT; i++) P3_Dial[i]		= (switchstate)P3_DialInit[i];

	// Panel 4
	int P4_SwitchInit[P4_SWITCHCOUNT]					= { 2, 2, 0, 0};
	for (i = 0; i < P4_SWITCHCOUNT; i++) P4_Switch[i]	= (switchstate)P4_SwitchInit[i];

	// Panel 5
	int P5_SwitchInit[P5_SWITCHCOUNT]					= { 2, 0, 1, 0, 1, 0, 1, 0};
	for (i = 0; i < P5_SWITCHCOUNT; i++) P5_Switch[i]	= (switchstate)P5_SwitchInit[i];

	// Panel 6
	int P6_SwitchInit[P6_SWITCHCOUNT]					= { 0, 0, 0, 0, 0, 0, 0, 0};
	for (i = 0; i < P6_SWITCHCOUNT; i++) P6_Switch[i]	= (switchstate)P6_SwitchInit[i];
	
	int P6_DialInit[P6_DIALCOUNT]						= { 0, 0, 0, 0};
	for (i = 0; i < P6_DIALCOUNT; i++) P6_Dial[i]		= (switchstate)P6_DialInit[i];

	// Panel 8
	int P8_SwitchInit[P8_SWITCHCOUNT]					= { 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1};
	for (i = 0; i < P8_SWITCHCOUNT; i++) P8_Switch[i]	= (switchstate)P8_SwitchInit[i];

	// Panel 12
	int P12_SwitchInit[P12_SWITCHCOUNT]					= { 1, 2, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1};
	for (i = 0; i < P12_SWITCHCOUNT; i++) P12_Switch[i]	= (switchstate)P12_SwitchInit[i];

	int P12_DialInit[P12_DIALCOUNT]						= { 5, 6, 0, 0};
	for (i = 0; i < P12_DIALCOUNT; i++) P12_Dial[i]		= (switchstate)P12_DialInit[i];

	// Panel 14
	int P14_SwitchInit[P14_SWITCHCOUNT]					= { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	for (i = 0; i < P14_SWITCHCOUNT; i++) P14_Switch[i]	= (switchstate)P14_SwitchInit[i];	

	int P14_DialInit[P14_DIALCOUNT]						= { 0};
	for (i = 0; i < P14_DIALCOUNT; i++) P14_Dial[i]		= (switchstate)P14_DialInit[i];

	// Load VC display source
	sh_displaysource	= oapiLoadTexture ("AAPO/PanelElements/LEM_instruments.dds");
}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
LM_COCKPIT::~LM_COCKPIT ()
{
	int i = 0;

	// Only you can prevent memory leaks
	for (i = 0; i < LM_VC_NEEDLECOUNT; i++) delete mgt_Needle[i];
	
	for (i = 0; i < P1_SWITCHCOUNT; i++) delete mgt_P1switch[i];
	for (i = 0; i < P1_DIALCOUNT; i++) delete mgt_P1Dial[i];

	for (i = 0; i < P2_SWITCHCOUNT; i++) delete mgt_P2switch[i];
	for (i = 0; i < P2_DIALCOUNT; i++) delete mgt_P2Dial[i];

	for (i = 0; i < P3_SWITCHCOUNT; i++) delete mgt_P3switch[i];
	for (i = 0; i < P3_DIALCOUNT; i++) delete mgt_P3Dial[i];

	for (i = 0; i < P4_SWITCHCOUNT; i++) delete mgt_P4switch[i];

	for (i = 0; i < P5_SWITCHCOUNT; i++) delete mgt_P5switch[i];

	for (i = 0; i < P6_SWITCHCOUNT; i++) delete mgt_P6switch[i];
	for (i = 0; i < P6_DIALCOUNT; i++) delete mgt_P6Dial[i];

	for (i = 0; i < P8_SWITCHCOUNT; i++) delete mgt_P8switch[i];

	for (i = 0; i < P12_SWITCHCOUNT; i++) delete mgt_P12switch[i];
	for (i = 0; i < P12_DIALCOUNT; i++) delete mgt_P12Dial[i];

	for (i = 0; i < P14_SWITCHCOUNT; i++) delete mgt_P14switch[i];
	for (i = 0; i < P14_DIALCOUNT; i++) delete mgt_P14Dial[i];
}

// ==============================================================
// Public functions
// ==============================================================

// --------------------------------------------------------------
// Initialize VC
// --------------------------------------------------------------
void LM_COCKPIT::InitVC (UINT mesh, LM_SYSTEMS *sys)
{
	int i = 0;
	lastredraw = 0;
	lm = sys;
	mh_cockpit = v->mh_cockpit;
	OrbiterSoundID = v->OrbiterSoundID;

	// Define tape gauge animations
	static UINT meshgroup_Gauges[LM_VC_NEEDLECOUNT];

	for ( i = 0; i < LM_VC_NEEDLECOUNT; i++)
	{
		meshgroup_Gauges[i] = VC_GRP_Panel_Needle + i;

		mgt_Needle[i] = new MGROUP_TRANSLATE (mesh, &meshgroup_Gauges[i], 1, _V( 0.00, 0.061*cos(P1_TILT), 0.061*sin(P1_TILT)));
		anim_TapeGuage[i] = v->CreateAnimation (0.0);
		v->AddAnimationComponent (anim_TapeGuage[i], 0.0f, 1.0f, mgt_Needle[i]);
	}

	// Define panel 1 animations 
	static UINT meshgroup_P1switches[P1_SWITCHCOUNT], meshgroup_P1dials[P1_DIALCOUNT];
	for (int i = 0; i < P1_SWITCHCOUNT; i++)
	{
		meshgroup_P1switches[i] = (VC_GRP_Switch_P1_00 + i);

		mgt_P1switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P1switches[i], 1, P1_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P1switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P1switch[i], 0.0f, 1.0f, mgt_P1switch[i]);
	}

	for ( i = 0; i < P1_DIALCOUNT; i++)
	{
		 meshgroup_P1dials[i] = VC_GRP_Dial_P1_01 + i;
		
		mgt_P1Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P1dials[i], 1, P1_DIAL_POS[i], P1_DIAL_AXIS, (float) 2*PI);
		anim_P1_Dial[i] = v->CreateAnimation (0.00);
		v->AddAnimationComponent (anim_P1_Dial[i], 0.0f, 1.0f, mgt_P1Dial[i]);
	}

	// Define panel 2 animations
	static UINT meshgroup_P2switches[P2_SWITCHCOUNT], meshgroup_P2dials[P2_DIALCOUNT];

	for (int i = 0; i < P2_SWITCHCOUNT; i++)
	{
		meshgroup_P2switches[i] = VC_GRP_Switch_P2_01 + i;
	
		mgt_P2switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P2switches[i], 1, P2_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P2switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P2switch[i], 0.0f, 1.0f, mgt_P2switch[i]);
	}

	for ( i = 0; i < P2_DIALCOUNT; i++)
	{
		 meshgroup_P2dials[i] = VC_GRP_Dial_P2_01 + i;
		
		mgt_P2Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P2dials[i], 1, P2_DIAL_POS[i], P2_DIAL_AXIS, (float) 2*PI);
		anim_P2_Dial[i] = v->CreateAnimation (0.00);
		v->AddAnimationComponent (anim_P2_Dial[i], 0.0f, 1.0f, mgt_P2Dial[i]);
	}

	// Define panel 3 animations
	static UINT meshgroup_P3switches[P3_SWITCHCOUNT], meshgroup_P3dials[P3_DIALCOUNT];
	for (int i = 0; i < P3_SWITCHCOUNT; i++)
	{
		meshgroup_P3switches[i] = VC_GRP_Switch_P3_01 + i;
	
		mgt_P3switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P3switches[i], 1, P3_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P3switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P3switch[i], 0.0f, 1.0f, mgt_P3switch[i]);
	}

	for ( i = 0; i < 6; i++)
	{
		meshgroup_P3dials[i] = VC_GRP_Dial_P3_01 + i;
		
		mgt_P3Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P3dials[i], 1, P3_DIAL_POS[i], P3_DIAL_AXIS, (float) 2*PI);
		anim_P3_Dial[i] = v->CreateAnimation (0.00);
		v->AddAnimationComponent (anim_P3_Dial[i], 0.0f, 1.0f, mgt_P3Dial[i]);
	}

	// Define panel 4 animations
	static UINT meshgroup_P4switches[P4_SWITCHCOUNT];
	for (int i = 0; i < P4_SWITCHCOUNT; i++)
	{
		meshgroup_P4switches[i] = VC_GRP_Switch_P4_01 + i;
	
		mgt_P4switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P4switches[i], 1, P4_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P4switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P4switch[i], 0.0f, 1.0f, mgt_P4switch[i]);
	}

	// Define panel 5 animations
	static UINT meshgroup_P5switches[P5_SWITCHCOUNT];
	for (int i = 0; i < P5_SWITCHCOUNT; i++)
	{
		meshgroup_P5switches[i] = VC_GRP_Switch_P5_01 + i;
	
		mgt_P5switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P5switches[i], 1, P5_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P5switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P5switch[i], 0.0f, 1.0f, mgt_P5switch[i]);
	}

	// Define panel 6 animations
	static UINT meshgroup_P6switches[P6_SWITCHCOUNT], meshgroup_P6dials[P6_DIALCOUNT];

	for (int i = 0; i < P6_SWITCHCOUNT; i++)
	{
		meshgroup_P6switches[i] = VC_GRP_Switch_P6_00 + i;
	
		mgt_P6switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P6switches[i], 1, P6_TOGGLE_POS[i], _V( 1, 0, 0), (float)PI/4);
		anim_P6switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P6switch[i], 0.0f, 1.0f, mgt_P6switch[i]);
	}

	for ( i = 0; i < P6_DIALCOUNT; i++)
	{
		meshgroup_P6dials[i] = VC_GRP_Dial_P6_00 + i;

		mgt_P6Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P6dials[i], 1, P6_DIAL_POS[i], P6_DIAL_AXIS, (float) 2*PI);
		anim_P6_Dial[i] = v->CreateAnimation (0.00);
		v->AddAnimationComponent ( anim_P6_Dial[i], 0.0f, 1.0f, mgt_P6Dial[i]);
	}

	// Define panel 8 animations
	static UINT meshgroup_P8switches[P8_SWITCHCOUNT];
	for (int i = 0; i < P8_SWITCHCOUNT; i++)
	{
		meshgroup_P8switches[i] = VC_GRP_Switch_P8_01 + i;
	
		mgt_P8switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P8switches[i], 1, P8_TOGGLE_POS[i], _V( 0, 0, 1), (float)PI/4);
		anim_P8switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P8switch[i], 0.0f, 1.0f, mgt_P8switch[i]);
	}

	// Define panel 12 animations
	static UINT meshgroup_P12switches[P12_SWITCHCOUNT], meshgroup_P12dials[P12_DIALCOUNT];

	for (int i = 0; i < P12_SWITCHCOUNT; i++)
	{
		meshgroup_P12switches[i] = VC_GRP_Switch_P12_01 + i;
	
		mgt_P12switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P12switches[i], 1, P12_TOGGLE_POS[i], _V( 0, 0,-1), (float)PI/4);
		anim_P12switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P12switch[i], 0.0f, 1.0f, mgt_P12switch[i]);
	}
	
	for ( i = 0; i < P12_DIALCOUNT; i++)
	{
		meshgroup_P12dials[i] = VC_GRP_Dial_P12_01 + i;

		mgt_P12Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P12dials[i], 1, P12_DIAL_POS[i], P12_DIAL_AXIS, (float) 2*PI);
		anim_P12_Dial[i] = v->CreateAnimation(0.00);
		v->AddAnimationComponent (anim_P12_Dial[i], 0.0f, 1.0f, mgt_P12Dial[i]);
	}

	// Define panel 14 animations
	static UINT meshgroup_P14switches[P14_SWITCHCOUNT], meshgroup_P14dials[P14_DIALCOUNT];

	for (int i = 0; i < P14_SWITCHCOUNT; i++)
	{
		meshgroup_P14switches[i] = VC_GRP_Switch_P14_01 + i;
	
		mgt_P14switch[i] = new MGROUP_ROTATE (mesh, &meshgroup_P14switches[i], 1, P14_TOGGLE_POS[i], _V( 0, 0,-1), (float)PI/4);
		anim_P14switch[i] = v->CreateAnimation (0.5);
		v->AddAnimationComponent (anim_P14switch[i], 0.0f, 1.0f, mgt_P14switch[i]);
	}

	for ( i = 0; i < P14_DIALCOUNT; i++)
	{
		meshgroup_P14dials[i] = VC_GRP_Dial_P14_01 + i;

		mgt_P14Dial[i] = new MGROUP_ROTATE (mesh, & meshgroup_P14dials[i], 1, P14_DIAL_POS[i], P14_DIAL_AXIS, (float) 2*PI);
		anim_P14_Dial[i] = v->CreateAnimation(0.00);
		v->AddAnimationComponent (anim_P14_Dial[i], 0.0f, 1.0f, mgt_P14Dial[i]);
	}

} // End "LM_COCKPIT::InitVC"

// --------------------------------------------------------------
// Read status from scenario file
// --------------------------------------------------------------
bool LM_COCKPIT::ParseScenarioLine (char *line)
{
	int i = 0;
	int state = 0;

	// Load Panel 1 switch positions
	if (!_strnicmp (line, "PANEL1_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P1_SWITCHCOUNT) && (*line); i++) P1_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	else if (!_strnicmp (line, "PANEL1_DIALS", 12))
	{
		line += 13;
		for (i = 0; (i < P1_DIALCOUNT) && (*line); i++) P1_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 2 switch positions
	else if (!_strnicmp (line, "PANEL2_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P2_SWITCHCOUNT) && (*line); i++) P2_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	else if (!_strnicmp (line, "PANEL2_DIALS", 12))
	{
		line += 13;
		for (i = 0; (i < P2_DIALCOUNT) && (*line); i++) P2_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 3 switch positions
	else if (!_strnicmp (line, "PANEL3_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P3_SWITCHCOUNT) && (*line); i++) P3_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	else if (!_strnicmp (line, "PANEL3_DIALS", 12))
	{
		line += 13;
		for (i = 0; (i < P3_DIALCOUNT) && (*line); i++) P3_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 4 switch positions
	else if (!_strnicmp (line, "PANEL4_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P4_SWITCHCOUNT) && (*line); i++) P4_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 5 switch positions
	else if (!_strnicmp (line, "PANEL5_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P5_SWITCHCOUNT) && (*line); i++) P5_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}
	
	// Load panel 6 switch positions
	else if (!_strnicmp (line, "PANEL6_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P6_SWITCHCOUNT) && (*line); i++) P6_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	else if (!_strnicmp (line, "PANEL6_DIALS", 12))
	{
		line += 13;
		for (i = 0; (i < P6_DIALCOUNT) && (*line); i++) P6_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}


	// Load Panel 8 switch positions
	else if (!_strnicmp (line, "PANEL8_SWITCHES", 15)) 
	{
		line += 16;
		for (i = 0; (i < P8_SWITCHCOUNT) && (*line); i++) P8_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 12 switch positions
	else if (!_strnicmp (line, "PANEL12_SWITCHES", 16)) 
	{
		line += 17;
		for (i = 0; (i < P12_SWITCHCOUNT) && (*line); i++) P12_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}
		
	else if (!_strnicmp (line, "PANEL12_DIALS", 13))
	{
		line += 14;
		for (i = 0; (i < P12_DIALCOUNT) && (*line); i++) P12_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}

	// Load Panel 14 switch positions
	else if (!_strnicmp (line, "PANEL14_SWITCHES", 16)) 
	{
		line += 17;
		for (i = 0; (i < P14_SWITCHCOUNT) && (*line); i++) P14_Switch[i] = (switchstate)(*line++ - '0');
		return true;
	}

	else if (!_strnicmp (line, "PANEL14_DIALS", 13))
	{
		line += 14;
		for (i = 0; (i < P14_DIALCOUNT) && (*line); i++) P14_Dial[i] = (switchstate)(*line++ - '0');
		return true;
	}	

	else return false;
} // End "LM_COCKPIT::LoadState"

// --------------------------------------------------------------
// Write status to scenario file
// --------------------------------------------------------------
void LM_COCKPIT::SaveState (FILEHANDLE scn)
{
	int i = 0;
	char switchpos[256];

	// Save Panel 1 state
	for (i = 0; i < P1_SWITCHCOUNT; i++) switchpos[i] = P1_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL1_SWITCHES", switchpos);

	for (i = 0; i < P1_DIALCOUNT; i++) switchpos[i] = P1_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL1_DIALS", switchpos);

	// Save Panel 2 state
	for (i = 0; i < P2_SWITCHCOUNT; i++) switchpos[i] = P2_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL2_SWITCHES", switchpos);

	for (i = 0; i < P2_DIALCOUNT; i++) switchpos[i] = P2_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL2_DIALS", switchpos);

	// Save Panel 3 state
	for (i = 0; i < P3_SWITCHCOUNT; i++) switchpos[i] = P3_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL3_SWITCHES", switchpos);

	for (i = 0; i < P3_DIALCOUNT; i++) switchpos[i] = P3_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL3_DIALS", switchpos);

	// Save Panel 4 state
	for (i = 0; i < P4_SWITCHCOUNT; i++) switchpos[i] = P4_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL4_SWITCHES", switchpos);

	// Save Panel 5 state
	for (i = 0; i < P5_SWITCHCOUNT; i++) switchpos[i] = P5_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL5_SWITCHES", switchpos);

	// Save Panel 6 state
	for (i = 0; i < P6_SWITCHCOUNT; i++) switchpos[i] = P6_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL6_SWITCHES", switchpos);

	for (i = 0; i < P6_DIALCOUNT; i++) switchpos[i] = P6_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL6_DIALS", switchpos);

	// Save Panel 8 state
	for (i = 0; i < P8_SWITCHCOUNT; i++) switchpos[i] = P8_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL8_SWITCHES", switchpos);

	// Save Panel 12 state
	for (i = 0; i < P12_SWITCHCOUNT; i++) switchpos[i] = P12_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL12_SWITCHES", switchpos);

	for (i = 0; i < P12_DIALCOUNT; i++) switchpos[i] = P12_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL12_DIALS", switchpos);

	// Save Panel 14 state
	for (i = 0; i < P14_SWITCHCOUNT; i++) switchpos[i] = P14_Switch[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL14_SWITCHES", switchpos);

	for (i = 0; i < P14_DIALCOUNT; i++) switchpos[i] = P14_Dial[i] + '0';
	switchpos[i] = '\0';
	oapiWriteScenario_string(scn, "PANEL14_DIALS", switchpos);


} // End "LM_COCKPIT::SaveState"

// --------------------------------------------------------------
// Pre-Step processes
// --------------------------------------------------------------
void LM_COCKPIT::PreStep (double simt, double simdt, double mjd)
{		

} // End "LM_COCKPIT::PreStep"

// --------------------------------------------------------------
// Post-Step processes
// --------------------------------------------------------------
void LM_COCKPIT::PostStep (double simt, double simdt, double mjd)
{		
	if (oapiGetFocusInterface() == v) // secure hud calls to prevent conflicts with other vessels or UMMUs
	{
		// Update Digital displays
		if (simt - lastredraw > 1) // If time since last update is greater than update interval...
		{
			// update displays
			oapiTriggerRedrawArea (0, -1, AID_LCD_FUELQTY);
			oapiTriggerRedrawArea (0, -1, AID_LCD_RCSQTY);
			oapiTriggerRedrawArea (0, -1, AID_LCD_DELTAV);

			// Reset interval
			lastredraw = simdt; 
		}

		// Update Date and Time displays
		if (fabs (mjd - lm->Display_Date) > 1)
		{
			lm->Display_Date = (int)mjd;
			oapiTriggerRedrawArea (0, -1, AID_LCD_DATE);
		}

		if (fabs (lm->MissionTime - lm->Display_Clock) > 1)
		{
			lm->Display_Clock = (int)lm->MissionTime;
			oapiTriggerRedrawArea (0, -1, AID_LCD_CLOCK);
		}

		if (fabs (lm->EventTime - lm->Display_Timer) > 1)
		{
			lm->Display_Timer = (int)lm->EventTime;
			oapiTriggerRedrawArea (0, -1, AID_LCD_TIMER);
		}
	} // End "if (oapiGetFocusInterface())"
} // End "LM_COCKPIT::PostStep"

// --------------------------------------------------------------
// Register VC active areas
// --------------------------------------------------------------
void LM_COCKPIT::RegisterActiveAreas (VECTOR3 ofs)
{
	int i = 0;

	// Get display surfaces from VC mesh 
	SURFHANDLE sh_PanelElements = oapiGetTextureHandle (mh_cockpit, 1);
	SURFHANDLE sh_alt = oapiGetTextureHandle (mh_cockpit, 7);
	SURFHANDLE sh_vs = oapiGetTextureHandle (mh_cockpit, 8);

	// EVA hatch handle
	oapiVCRegisterArea (AID_EVAHANDLE_OPEN, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical( AID_EVAHANDLE_OPEN, _V(-0.3356, -0.6232, 1.5988) + ofs, 0.08);

	oapiVCRegisterArea (AID_EVAHANDLE_CLOSE, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_EVAHANDLE_CLOSE, _V( 0.3300, -0.4888, 0.9368) + ofs, 0.08);

	// Caution and warning lights
	oapiVCRegisterArea (AID_CAUTIONADVISORY, _R(7,244,605,359), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND, sh_PanelElements);

	oapiVCRegisterArea (AID_P1_MASTERCAUTION,  _R( 612, 239, 676, 303), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN, PANEL_MAP_BACKGROUND, sh_PanelElements);
	oapiVCSetAreaClickmode_Spherical( AID_P1_MASTERCAUTION, MASTERCAUTION_POS[0] + ofs, 0.02);
	
	oapiVCRegisterArea (AID_P2_MASTERCAUTION,  _R( 612, 239, 676, 303), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN, PANEL_MAP_BACKGROUND, sh_PanelElements);
	oapiVCSetAreaClickmode_Spherical( AID_P2_MASTERCAUTION, MASTERCAUTION_POS[1] + ofs, 0.02);

	// Main panel digital displays
	oapiVCRegisterArea (AID_LCD_DATE, _R( 0, 896, 216, 936), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);
	oapiVCRegisterArea (AID_LCD_CLOCK, _R( 0, 936, 216, 976), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);
	oapiVCRegisterArea (AID_LCD_TIMER, _R( 0, 976, 216, 1024), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);

	oapiVCRegisterArea (AID_LCD_FUELQTY, _R(216,896,312,1024), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);
	oapiVCRegisterArea (AID_LCD_RCSQTY, _R(312,896,408,976), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);
	oapiVCRegisterArea (AID_LCD_DELTAV, _R(312,976,456,1016), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_PanelElements);
	
	// Main panel guages
	oapiVCRegisterArea (AID_GAUGE_ALTRANGE, _R(0,1753,64,2009), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_alt);
	oapiVCRegisterArea (AID_GAUGE_RANGERATE, _R(0,512,64,768), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_NONE, sh_vs);

	for (i = 0; i < LM_VC_NEEDLECOUNT; i++)	
	{
		oapiVCRegisterArea (AID_GAUGE_MAINTHRUST + i, PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE);
	}

	// Pilot and Commander ADIs

	// Abort and stage seperation push-buttons
	oapiVCRegisterArea (AID_BTN_P1_ABORT, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_BTN_P1_ABORT, UNDOCK_BTN_POS + ofs, 0.02);

	oapiVCRegisterArea (AID_BTN_P1_ABORTSTAGE, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_BTN_P1_ABORTSTAGE, ABORTSTAGE_BTN_POS + ofs, 0.022);

	// X-translate (Ullage Burn) push-buttons
	oapiVCRegisterArea (AID_BTN_P5_XTRANS, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_BTN_P5_XTRANS, XTRANS_BTN_POS + ofs, 0.022);

	// Main engine start and stop push-buttons
	oapiVCRegisterArea (AID_BTN_P5_START, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_BTN_P5_START, START_BTN_POS + ofs, 0.022);

	oapiVCRegisterArea (AID_BTN_P5_STOP, PANEL_REDRAW_NEVER, PANEL_MOUSE_DOWN);
	oapiVCSetAreaClickmode_Spherical (AID_BTN_P5_STOP, STOP_BTN_POS + ofs, 0.022);

	// Pilot's Keypad

	// MFD Buttons
	VECTOR3 MFD_buttons [11];
	VECTOR3 MFD_position = MFD_POS + ofs;																				// Location of MFD within VC
	for (i = 0; i < 11; i++) MFD_buttons[i] = MFD_position/* + RotateVector (MFD_BUTTON_POS[i], RAD*45, _V( 1, 0, 0))*/;	// Rotate button coordinates to match panel tilt

	oapiVCRegisterArea (AID_MFD0_LBUTTONS, _R( 0, 96, 32, 240), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN, PANEL_MAP_BACKGROUND, sh_PanelElements);
	oapiVCSetAreaClickmode_Quadrilateral (AID_MFD0_LBUTTONS, MFD_buttons[0], MFD_buttons[1], MFD_buttons[2], MFD_buttons[3]);

	oapiVCRegisterArea (AID_MFD0_RBUTTONS, _R( 32, 96, 64, 240), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN, PANEL_MAP_BACKGROUND, sh_PanelElements);
	oapiVCSetAreaClickmode_Quadrilateral (AID_MFD0_RBUTTONS, MFD_buttons[4], MFD_buttons[5], MFD_buttons[6], MFD_buttons[7]);

	oapiVCRegisterArea (AID_MFD0_PWR, _R(0,0,0,0), PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN, PANEL_MAP_NONE, NULL);
	oapiVCSetAreaClickmode_Spherical (AID_MFD0_PWR, MFD_buttons[8], 0.01);

	oapiVCRegisterArea (AID_MFD0_SEL, _R(0,0,0,0), PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN, PANEL_MAP_NONE, NULL);
	oapiVCSetAreaClickmode_Spherical (AID_MFD0_SEL, MFD_buttons[9], 0.01);

	oapiVCRegisterArea (AID_MFD0_MNU, _R(0,0,0,0), PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN, PANEL_MAP_NONE, NULL); 
	oapiVCSetAreaClickmode_Spherical (AID_MFD0_MNU, MFD_buttons[10], 0.01);

	// Panel 1 switches
	for (i = 0; i < P1_SWITCHCOUNT; i++)	
	{
		oapiVCRegisterArea (AID_SWITCH_P1_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P1_00 + i, P1_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P1_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P1_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P1_00 + i, P1_DIAL_POS[i] + ofs, 0.02);
	}

	// Panel 2 switches
	for (i = 0; i < P2_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P2_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P2_00 + i, P2_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P2_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P2_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P2_00 + i, P2_DIAL_POS[i] + ofs, 0.02);
	}

	// Panel 3 switches
	for (i = 0; i < P3_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P3_01 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P3_01 + i, P3_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P3_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P3_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P3_00 + i, P3_DIAL_POS[i] + ofs, 0.02);		
	}

	// Panel 4 switches
	for (i = 0; i < P4_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P4_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P4_00 + i, P4_TOGGLE_POS[i] + ofs, 0.012);
	}

	// Panel 5 switches
	for (i = 0; i < P5_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P5_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P5_00 + i, P5_TOGGLE_POS[i] + ofs, 0.012);
	}

	// Panel 6 switches
	for (i = 0; i < P6_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P6_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P6_00 + i, P6_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P6_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P6_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P6_00 + i, P6_DIAL_POS[i] + ofs, 0.02);
	}

	// Panel 8 switches
	for (i = 0; i < P8_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P8_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P8_00 + i, P8_TOGGLE_POS[i] + ofs, 0.012);
	}
	
	// Panel 12 switches
	for (i = 0; i < P12_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P12_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P12_00 + i, P12_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P12_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P12_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P12_00 + i, P12_DIAL_POS[i] + ofs, 0.02);
	}

	// Panel 14 switches
	for (i = 0; i < P14_SWITCHCOUNT; i++)
	{
		oapiVCRegisterArea (AID_SWITCH_P14_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_SWITCH_P14_00 + i, P14_TOGGLE_POS[i] + ofs, 0.012);
	}

	for (i = 0; i < P14_DIALCOUNT; i++)
	{
		oapiVCRegisterArea (AID_DIAL_P14_00 + i, PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN);
		oapiVCSetAreaClickmode_Spherical (AID_DIAL_P14_00 + i, P14_DIAL_POS[i] + ofs, 0.02);
	}

} // End "LM_COCKPIT::RegisterActiveAreas"

// --------------------------------------------------------------
// Respond to user inputs
// --------------------------------------------------------------
bool LM_COCKPIT::MouseEvent (int id, int event, VECTOR3 &p)
{
	switch (id) 
	{
	// EVA hatch handle
	case AID_EVAHANDLE_OPEN:
		if (v->HatchStatus == CLOSED) v->ActivateHatch (); // If hatch is closed activate hatch
		return true;

	case AID_EVAHANDLE_CLOSE:
		if (v->HatchStatus == OPEN) v->ActivateHatch (); // If hatch is open activate hatch
		return true;

	// Master Caution lights
	case AID_P1_MASTERCAUTION:
	case AID_P2_MASTERCAUTION:
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		if (MasterCautionFlag) MasterCautionFlag = false;
		oapiTriggerRedrawArea (0, -1, AID_P1_MASTERCAUTION);
		oapiTriggerRedrawArea (0, -1, AID_P2_MASTERCAUTION);
		return true;

	// Abort push-button
	case AID_BTN_P1_ABORT: 
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		return true;

	// Abort Stage push-button
	case AID_BTN_P1_ABORTSTAGE: 
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		return true;

	// X-translate (Ullage Burn) push-button
	case AID_BTN_P5_XTRANS:
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		return true;

	// Main engine start and stop push-buttons
	case AID_BTN_P5_START:
		lm->EngineStart ();
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		return true;

	case AID_BTN_P5_STOP:
		lm->KillThrust ();
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		return true;

	// MFD Buttons
	case AID_MFD0_LBUTTONS:	// Left side
	case AID_MFD0_RBUTTONS:	// Right side
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		double dp;								// Because MFD buttons are defined as a group we must break it down to determine which specific button was pressed.
		if (modf (p.y*23.0/4.0, &dp) < 0.75)	// by casting "dp" to an "int" we get a "rounded out" value for the click point's vertical position. This value will corrispond to 1 of the 6 buttons on each side of the MFD.
		{
			int bt = (int)dp + (id == AID_MFD0_LBUTTONS ? 0 : 6);	// Because rounding out only accounts for 6 of the 12 buttons, we add either 0 or 6 to 'dp' depending on which side of the screen the mouse is on.
			oapiProcessMFDButton (CmdMFD, bt, event);				// 'bt' now returns a value between 1 and 12 corrisponding to one of the MFD's 12 buttons, OAPI does it's thing.
			return true;											// we're done
		}
		break;

	case AID_MFD0_PWR:		// Power
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		oapiToggleMFD_on (CmdMFD);
		return true;

	case AID_MFD0_SEL:		// Select
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		oapiSendMFDKey (CmdMFD, OAPI_KEY_F1);
		return true;

	case AID_MFD0_MNU:		// Menu
		PlayVesselWave (OrbiterSoundID, SOUND_BTN);
		oapiSendMFDKey (CmdMFD, OAPI_KEY_GRAVE);
		return true;

	// Panel Switches
	default:
		if (AID_PANEL_1) return Switches_Panel1 (id, event);
		if (AID_PANEL_2) return Switches_Panel2 (id, event);
		if (AID_PANEL_3) return Switches_Panel3 (id, event);
		if (AID_PANEL_4) return Switches_Panel4 (id, event);
		if (AID_PANEL_5) return Switches_Panel5 (id, event);
		if (AID_PANEL_6) return Switches_Panel6 (id, event);
		if (AID_PANEL_8) return Switches_Panel8 (id, event);
		if (AID_PANEL_12) return Switches_Panel12 (id, event);
		if (AID_PANEL_14) return Switches_Panel14 (id, event);
	} // End "switch (id)"

	return false;
} // End "LM_COCKPIT::ProcessMouseEvent"

// --------------------------------------------------------------
// Respond to redraw requests
// --------------------------------------------------------------
bool LM_COCKPIT::RedrawEvent (int id, int event, SURFHANDLE surf)
{
	switch (id)
	{
	// Main panel displays
	case AID_CAUTIONADVISORY:
		return Redraw_CautionAdvisory (surf);

	case AID_P1_MASTERCAUTION:
	case AID_P2_MASTERCAUTION:
		return Redraw_MasterCaution (surf);

	case AID_LCD_DATE:
		return Redraw_Date (surf, lm->Display_Date);

	case AID_LCD_CLOCK:
		return Redraw_Clocks (surf, lm->Display_Clock);

	case AID_LCD_TIMER:
		return Redraw_Clocks (surf, lm->Display_Timer);

	case AID_LCD_FUELQTY:
		return Redraw_MainFuelQuantity (surf);

	case AID_LCD_DELTAV:
		return Redraw_DeltaV (surf);

	case AID_LCD_RCSQTY:
		return Redraw_RCSFuelQuantity (surf);

	// MFD Buttons
	case AID_MFD0_LBUTTONS:
 		return Redraw_MFDButtons (surf, 0, 0);

	case AID_MFD0_RBUTTONS:
		return Redraw_MFDButtons (surf, 0, 1);

	// Analog Gauges:
	case AID_GAUGE_MAINTHRUST:
	case AID_GAUGE_MAINTEMP:
	case AID_GAUGE_MAINPRESS:
		return Redraw_Panel1Gauges ();

	case AID_GAUGE_RCSTEMP:
	case AID_GAUGE_RCSPRESS:
	case AID_GAUGE_ECSTEMP:
	case AID_GAUGE_ECSPRESS:
	case AID_GAUGE_ECSCO2LVL:
	case AID_GAUGE_ECSGLYCOL:
	case AID_GAUGE_ECSQTY:
		return Redraw_Panel2Gauges ();

	default:
		return Redraw_PanelSwitches (id, event);
	}
} // End "LM_COCKPIT::RedrawEvent"

// --------------------------------------------------------------
// Master caution alarm
// --------------------------------------------------------------
bool LM_COCKPIT::TripMasterCaution (void)
{
	if(!MasterCautionFlag) // if alarm has not already been tripped
	{
		// Trip alarm
		MasterCautionFlag = true;

		// Redraw panel master caution lights
		oapiTriggerRedrawArea (0, -1, AID_P1_MASTERCAUTION);
		oapiTriggerRedrawArea (0, -1, AID_P2_MASTERCAUTION);

		// Set time acceleration to 1 when alarm is triggered.
		if ((oapiGetFocusInterface () == v) && (oapiGetTimeAcceleration () > 1)) oapiSetTimeAcceleration (1);
		return true;
	}
	else return false;
}

// ==============================================================
// Private functions
// ==============================================================

// --------------------------------------------------------------
// LCD print function (used to redraw main panel digital displays)
// --------------------------------------------------------------
void LM_COCKPIT::LCDPrint (SURFHANDLE surf, char* text, int row, int _length)
{
	int cHeight = 40;	// Character height [pixles]
	int cWidth = 24;	// Character width [pixels] 
	int cRow, cCol;		// Character's position in bitmap [row / column]
	int textwidth = cWidth * _length;

	int fontxofs = 0;
	int fontyofs = 15;

	for (int i = 0; i < strlen(text); ++i)
		{
		cRow = (text[i] - 32) / 30;
		cCol = (text[i] - 32) % 30;

		int srcX = fontxofs + cCol * cWidth;
		int srcY = fontyofs + cRow * cHeight;

		int tgtX = i * cWidth; 
		int	tgtY = row * cHeight; 

		oapiBlt (surf, sh_displaysource, tgtX, tgtY, srcX, srcY, cWidth, cHeight); // blit
		} // End "for (int i = 0; i < strlen(text); ++i)"

} // End "LM_COCKPIT::LCDPrint"

// --------------------------------------------------------------
// Update altitude/range indicator
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_Altitude (SURFHANDLE surf)
{
	return false;
} // End "LM_COCKPIT::Redraw_Altitude"

// --------------------------------------------------------------
// Update caution warning and advisory lights
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_CautionAdvisory (SURFHANDLE surf)
{
	// Constants
	const int CWA_x_ofs = 7;
	const int CWA_y_ofs = 244;
	const int CWA_ofs = 127;
	const int CWA_width = 70;	// width of each light in pixles
	const int CWA_height = 23;	// height of each light in pixles
	const int CWA_column[8] = { 0, 71, 156, 227, 311, 383, 458, 528};

	// Variables
	int iCol, iRow, src_x, src_y, tgt_x, tgt_y;

	// Caution Advisory Lights
	for (int i = 0; i < 40; i++) 
	{
		iCol = i / 5;
		tgt_x = CWA_column[iCol];
		src_x = tgt_x + CWA_x_ofs;

		iRow = i % 5;
		tgt_y = iRow * CWA_height;
		src_y = tgt_y + CWA_y_ofs;

		if (!CWA_Flag[i]) oapiBlt (surf, sh_displaysource, tgt_x, tgt_y, src_x, src_y, CWA_width, CWA_height);
		else oapiBlt (surf, sh_displaysource, tgt_x, tgt_y, src_x, (src_y + 127), CWA_width, CWA_height);
	}	

	return true;
} // End "LM_COCKPIT::Redraw_CautionAdvisory"


// --------------------------------------------------------------
// Update mission and event timers
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_Date (SURFHANDLE surf, double date)		
{
	VECTOR3 output = MJD_to_date (date);

	//if (lm->Panel1Power)
	//{
		LCDFormatPrint (surf, 0, " %2.0f%2.0f%4.0f", output.y, output.x, output.z);
	//}
	//else
	//{
		//LCDFormatPrint (surf, 0, "         ");
	//}

	return true;
} // End "LM_COCKPIT::Redraw_Clocks"

// --------------------------------------------------------------
// Update mission and event timers
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_Clocks (SURFHANDLE surf, double time)		
{

	VECTOR3 output = to_HrsMinSec (time);

	//if (lm->Panel1Power)
	//{
		LCDFormatPrint (surf, 0, " %4.0f%2.0f%2.0f", output.x, output.y, output.z);
	//}
	//else
	//{
		//LCDFormatPrint (surf, 0, "         ");
	//}

	return true;
} // End "LM_COCKPIT::Redraw_Clocks"

// --------------------------------------------------------------
// Update delta V display
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_DeltaV (SURFHANDLE surf)			
{
	// Calculate dV using Tsiolkovsky's rocket equation
	double ve = v->GetISP ();				// Rocket's exhaust velocity
	double m0 = v->GetMass ();				// Total mass of rocket	
	double m1 = m0 - v->GetFuelMass ();		// Mass of rocket at burnout
	double dv = ve * log (m0 / m1);			// Tsiolkovsky's equation

	//if (lm->Panel1Power)
	//{
		LCDFormatPrint (surf, 0, "%6.0f", dv);
		return true;
	//{
	// else return false;
} // End "LM_COCKPIT::Redraw_DeltaV"

// --------------------------------------------------------------
// Update fuel quantity display
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_MainFuelQuantity (SURFHANDLE surf)
{
	// Propellant and oxidizer quantities
	double O2	= lm->Display_Fuel_Qty * 0.6163; // Propellant display mass * Oxidizer mass ratio
	double prop	= lm->Display_Fuel_Qty * 0.3837; // Propellant display mass * fuel mass ratio	
	double He	= 0;

	if (SWITCH_PROPQTY_MON == DOWN) // if switch is in "OFF" position blank the display
	{
		LCDFormatPrint (surf, 0, "    ");
		LCDFormatPrint (surf, 1, "    ");
		LCDFormatPrint (surf, 2, "    ");
		return true;
	}
	else //...Otherwise
	{	
		LCDFormatPrint (surf, 0, "%4.0f", O2);		// Print Oxidizer mass
		LCDFormatPrint (surf, 1, "%4.0f", prop);	// Print Propellant mass
		LCDFormatPrint (surf, 2, "     ");			// Print pressurant remaining
		return true;
	}
	return false;
} // End "LM_COCKPIT::Redraw_MainFuelQuantity"

// --------------------------------------------------------------
// Update master caution lights
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_MasterCaution (SURFHANDLE surf)
{
	if (MasterCautionFlag) oapiBlt (surf, sh_displaysource, 0, 0, 612, 303, 64, 64);
	else oapiBlt (surf, sh_displaysource, 0, 0, 612, 239, 64, 64);

	return false;
} // End "LM_COCKPIT::Redraw_MasterCaution"

// --------------------------------------------------------------
// Update Panel 1 Tape gauges
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_Panel1Gauges (void)
{
	double simdt = oapiGetSimStep ();
	double	delta = simdt;	

	double lvl[6] = {lm->Display_Thrust, lm->Display_Throttle, lm->Display_Fuel_Temp, lm->Display_O2_Temp, lm->Display_Fuel_Press, lm->Display_O2_Press};
	 

	for (int i = 0; i < 6; i++)
	{	
		//if (!Panel1Power) lvl i = 0;

		if ((lvl[i] - gauge_proc[i]) > 0)
		{
			if ((gauge_proc[i] + delta) < lvl[i]) gauge_proc[i] += delta; 
			else gauge_proc[i] = lvl[i];
		}
		else if ((lvl[i] - gauge_proc[i]) < 0)
		{
			if ((gauge_proc[i] + delta) > lvl[i]) gauge_proc[i] -= delta; 
			else gauge_proc[i] = lvl[i];
		}

		v->SetAnimation(anim_TapeGuage[i], gauge_proc[i]);
	}
	return true;
}

// --------------------------------------------------------------
// Update Panel 2 Tape gauges
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_Panel2Gauges (void)
{
	double simdt = oapiGetSimStep ();
	double	delta = simdt;	

	double lvl[13] = {lm->Display_RCS_Temp[0], lm->Display_RCS_Temp[1], lm->Display_RCS_Press[0], lm->Display_RCS_Press[1], lm->Display_SuitTemp, lm->Display_CabinTemp, lm->Display_SuitPress, lm->Display_CabinPress, lm->Display_GlycolTemp, lm->Display_GlycolPress, lm->Display_O2_Qty, lm->Display_H2O_Qty,  lm->Display_CO2_Level};
	 
	for (int i = 0; i < 13; i++)
	{	
		//if (!Panel2Power) lvl i = 0;

		if ((lvl[i] - gauge_proc[i+6]) > 0)
		{
			if ((gauge_proc[i+6] + delta) < lvl[i]) gauge_proc[i+6] += delta; 
			else gauge_proc[i+6] = lvl[i];
		}
		else if ((lvl[i] - gauge_proc[i+6]) < 0)
		{
			if ((gauge_proc[i+6] + delta) > lvl[i]) gauge_proc[i+6] -= delta; 
			else gauge_proc[i+6] = lvl[i];
		}

		v->SetAnimation(anim_TapeGuage[i+6], gauge_proc[i+6]);
	}
	return true;
}

// --------------------------------------------------------------
// Update MFD button labels
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_MFDButtons (SURFHANDLE surf, int mfd, int side)
{
	int i, btn, len, w;
	const char *label;
	int ButtonW = 32;	// Button width in pixles
	int tgtX;			// target coordinates for bliting

	int btnrow[6] = { 4, 28, 52, 76, 100, 124};	// offset of MFD button rows in pixles

	// MFD label font
	const int CHX[256] = { // character position in LEM_Inst.dds (x-offset)
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,642/*+*/,0,661/*-*/,0,596/* / */,
		492/*0*/,501/*1*/,510/*2*/,520/*3*/,529/*4*/,538/*5*/,547/*6*/,556/*7*/,565/*8*/,575/*9*/,627/*:*/,621/*;*/,602/*<*/,652/*=*/,612/*>*/,0,
		0,1/*A*/,11/*B*/,21/*C*/,32/*D*/,43/*E*/,54/*F*/,63/*G*/,75/*H*/,86/*I*/,92/*J*/,101/*K*/,111/*L*/,120/*M*/,132/*N*/,143/*O*/,
		155/*P*/,165/*Q*/,176/*R*/,187/*S*/,198/*T*/,207/*U*/,218/*V*/,229/*W*/,242/*X*/,253/*Y*/,263/*Z*/,0,0,0,0,0,
		0,273/*a*/,282/*b*/,291/*c*/,299/*d*/,309/*e*/,318/*f*/,324/*g*/,333/*h*/,342/*i*/,347/*j*/,353/*k*/,362/*l*/,367/*m*/,380/*n*/,389/*o*/,
		398/*p*/,407/*q*/,416/*r*/,423/*s*/,431/*t*/,438/*u*/,447/*v*/,456/*w*/,466/*x*/,475/*y*/,483/*z*/,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	const int CHY = 0; // Character position in LEM_Instruments.dds (y-offset)

	const int CHW[256] = { // character widths
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,6/*+*/,0,4/*-*/,0,4/* / */,
		6/*0*/,6/*1*/,6/*2*/,6/*3*/,6/*4*/,6/*5*/,6/*6*/,6/*7*/,6/*8*/,6/*9*/,2/*:*/,2/*;*/,6/*<*/,6/*=*/,6/*>*/,0,
		0,8/*A*/,7/*B*/,7/*C*/,7/*D*/,6/*E*/,6/*F*/,8/*G*/,7/*H*/,2/*I*/,5/*J*/,7/*K*/,6/*L*/,8/*M*/,7/*N*/,8/*O*/,
		6/*P*/,8/*Q*/,7/*R*/,7/*S*/,6/*T*/,7/*U*/,8/*V*/,11/*W*/,7/*X*/,8/*Y*/,7/*Z*/,0,0,0,0,0,
		0,6/*a*/,6/*b*/,6/*c*/,6/*d*/,6/*e*/,4/*f*/,6/*g*/,6/*h*/,2/*i*/,3/*j*/,5/*k*/,2/*l*/,8/*m*/,6/*n*/,6/*o*/,
		6/*p*/,6/*q*/,4/*r*/,6/*s*/,4/*t*/,6/*u*/,6/*v*/,9/*w*/,6/*x*/,6/*y*/,6/*z*/,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	const int CHH = 12; // character height

	// because I broke the MFD's buttons into two seperate active areas (left and right) our redraw function has to differ between them.
	// there are 12 buttons total with 'side 0' (left) being the first 6 and 'side 1' being the last 6. 
	for (btn = 0 + side*6; btn < 6 + side*6; btn++)
	{
		if( label = oapiMFDButtonLabel (mfd, btn)) // Check if MFD button is active and get it's label as a string
		{
			len = strlen(label); // get label's string length
			for (w = i = 0; i < len; i++) w += CHW[label[i]]; // Calculate width of string in pixles 
			for (i = 0, tgtX = ButtonW/2 - w/2; i < len; i++) // for each character in the string...
			{
				w = CHW[label[i]]; // w = character width
				if (w) 
				{
					oapiBlt (surf, sh_displaysource, tgtX, btnrow[btn - side*6], CHX[label[i]], CHY, w, CHH); // blit character to target
					tgtX += w; // and shift tgtX 'w' pixles to the right
				}
			}
		}
		else break;
	}

	return true;
} // End "LM_COCKPIT::Redraw_MFDButtons"

// --------------------------------------------------------------
// Update panel switch positions
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_PanelSwitches (int id, int event)
{
	int i = 0;

	if (AID_PANEL_1)
	{
		
		for (i = 0; i < P1_SWITCHCOUNT; i++) v->SetAnimation (anim_P1switch[i], (float)P1_Switch[i]/2);

		v->SetAnimation (anim_P1_Dial[0], (float)P1_Dial[0]/6);
	
		return true;
	}

	else if (AID_PANEL_2)
	{
		for (i = 0; i < P2_SWITCHCOUNT; i++) v->SetAnimation (anim_P2switch[i], (float)P2_Switch[i]/2);
		
		v->SetAnimation (anim_P2_Dial[0], (float)P2_Dial[0]/12);
		v->SetAnimation (anim_P2_Dial[1], (float)P2_Dial[1]/6);		
		v->SetAnimation (anim_P2_Dial[2], (float)P2_Dial[2]/6);		
		v->SetAnimation (anim_P2_Dial[3], (float)P2_Dial[3]/12);

		return true;
	}

	else if (AID_PANEL_3)
	{
		for (i = 0; i < P3_SWITCHCOUNT; i++) v->SetAnimation (anim_P3switch[i], (float)P3_Switch[i]/2);

		v->SetAnimation (anim_P3_Dial[0], (float)P3_Dial[0]/12);
		v->SetAnimation (anim_P3_Dial[1], (float)P3_Dial[1]/12);
		v->SetAnimation (anim_P3_Dial[2], (float)P3_Dial[2]/12);
		v->SetAnimation (anim_P3_Dial[3], (float)P3_Dial[3]/12);
		v->SetAnimation (anim_P3_Dial[4], (float)P3_Dial[4]/12);
		v->SetAnimation (anim_P3_Dial[5], (float)P3_Dial[4]/12);
		
		return true;
	}

	else if (AID_PANEL_4)
	{
		for (i = 0; i < P4_SWITCHCOUNT; i++) v->SetAnimation (anim_P4switch[i], (float)P4_Switch[i]/2);
		return true;
	}

	else if (AID_PANEL_5)
	{
		for (i = 0; i < P5_SWITCHCOUNT; i++) v->SetAnimation (anim_P5switch[i], (float)P5_Switch[i]/2);
		return true;
	}

	else if (AID_PANEL_6)
	{
		for (i = 0; i < P6_SWITCHCOUNT; i++) v->SetAnimation (anim_P6switch[i], (float)P6_Switch[i]/2);
		
		v->SetAnimation (anim_P6_Dial[0], (float)P6_Dial[0]/12);
		v->SetAnimation (anim_P6_Dial[1], (float)P6_Dial[1]/12);
		v->SetAnimation (anim_P6_Dial[2], (float)P6_Dial[2]/12);		
		v->SetAnimation (anim_P6_Dial[3], (float)P6_Dial[3]/12);
		
		return true;
	}

	else if (AID_PANEL_8)
	{
		for (i = 0; i < P8_SWITCHCOUNT; i++) v->SetAnimation (anim_P8switch[i], (float)P8_Switch[i]/2);
		return true;
	}

	else if (AID_PANEL_12)
	{
		for (i = 0; i < P12_SWITCHCOUNT; i++) v->SetAnimation (anim_P12switch[i], (float)P12_Switch[i]/2);
	
		v->SetAnimation (anim_P12_Dial[0], (float)P12_Dial[0]/24 );
		v->SetAnimation (anim_P12_Dial[1], (float)P12_Dial[1]/24 );
		v->SetAnimation (anim_P12_Dial[2], (float)P12_Dial[2]/12 );		
		v->SetAnimation (anim_P12_Dial[3], (float)P12_Dial[3]/12 );
		
		return true;
	}

	else if (AID_PANEL_14)
	{
		for (i = 0; i < P14_SWITCHCOUNT; i++) v->SetAnimation (anim_P14switch[i], (float)P14_Switch[i]/2);
	
		v->SetAnimation (anim_P14_Dial[0], (float)P14_Dial[0]/12);
		
		return true;
	}

	else return false;
} // End "LM_COCKPIT::Redraw_PanelSwitches"

// --------------------------------------------------------------
// Update VS/Rate guage
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_RangeRate (SURFHANDLE surf)
{
	return false;
} // End "LM_COCKPIT::Redraw_RangeRate"

// --------------------------------------------------------------
// Update RCS fuel quantity display
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_RCSFuelQuantity (SURFHANDLE surf)
{
	double rcsa, rcsb; // quantity levels for RCS tanks "A" and "B"
	rcsa = lm->Display_RCS_Qty[0];
	rcsb = lm->Display_RCS_Qty[1];

	//if (lm->Panel2Power)
	//{
		if (rcsa >= 0.999) LCDFormatPrint (surf, 0, " 100")
		else LCDFormatPrint (surf, 0, "%3.1f", rcsa * 100)

		if (rcsb >= 0.999) LCDFormatPrint (surf, 1, " 100")
		else LCDFormatPrint (surf, 1, "%3.1f", rcsb * 100)
	//}
	//else
	//{
		//LCDFormatPrint (surf, 0, "    ");
		//LCDFormatPrint (surf, 1, "    ");
	//}

	return true;
} // End "LM_COCKPIT::Redraw_RCSFuelQuantity"

// --------------------------------------------------------------
// Update Commander's x-pointer
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_XpointerCMD (SURFHANDLE surf)
{
	return false;
} // End "LM_COCKPIT::Redraw_XpointerCMD"

// --------------------------------------------------------------
// Update Pilot's x-pointer
// --------------------------------------------------------------
bool LM_COCKPIT::Redraw_XpointerLMP (SURFHANDLE surf)
{
	return false;
} // End "LM_COCKPIT::Redraw_XpointerLMP"

// --------------------------------------------------------------
// 
// --------------------------------------------------------------
bool LM_COCKPIT::Switches_Panel1 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P1_00:
		return ActivateSwitch (event, &P1_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);

	case AID_SWITCH_P1_01:
		return ActivateSwitch (event, &P1_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_02:
		return ActivateSwitch (event, &P1_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_03: // Main Engine Arm switch [DES/OFF/ASC]
		return ActivateSwitch (event, &P1_Switch[3], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_04:
		return ActivateSwitch (event, &P1_Switch[4], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_05: // Throttle Control [MAN/AUTO]
		return ActivateSwitch (event, &P1_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_06:
		return ActivateSwitch (event, &P1_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_07:
		return ActivateSwitch (event, &P1_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_08: // Primary Manual Control [CDR/LMP]
		return ActivateSwitch (event, &P1_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_09:
		return ActivateSwitch (event, &P1_Switch[9], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_10:
		return ActivateSwitch (event, &P1_Switch[10], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_11: // Ascent Engine HeREG1 [OPEN/CLOSE]
		return ActivateSwitch (event, &P1_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_12: // Descent Engine HeREG1 [OPEN/CLOSE]
		return ActivateSwitch (event, &P1_Switch[12], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_13: // Ascent Engine HeREG2 [OPEN/CLOSE]
		return ActivateSwitch (event, &P1_Switch[13], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_14: // Descent Engine HeREG2 [OPEN/CLOSE]
		return ActivateSwitch (event, &P1_Switch[14], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_15: // Propellant Quantity Monitoring [ASC/DES/OFF]
		return ActivateSwitch (event, &P1_Switch[15], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_16:
		return ActivateSwitch (event, &P1_Switch[16], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_17:
		return ActivateSwitch (event, &P1_Switch[17], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_18:
		return ActivateSwitch (event, &P1_Switch[18], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P1_19:
		return ActivateSwitch (event, &P1_Switch[19], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_DIAL_P1_00:
		return ActivateSwitch (event, &P1_Dial[0], 5, SOUND_ROTARY, OrbiterSoundID);
		
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::ProcessMouseEvent"

bool LM_COCKPIT::Switches_Panel2 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P2_00: // RCS "A" propellant shut off valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_01: // RCS "A" Quad-1 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_02: // RCS "A" Quad-2 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_03: // RCS A/B cross-feed valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_04: // RCS "A" oxidizer shut off valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[4], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_05: // RCS "A" Quad-4 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_06: // RCS "A" Quad-3 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_07: // RCS "A" Ascent tank cross-feed valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_08: // RCS "B" propellant shut off valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_09: // RCS "B" Quad-2 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[9], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_10: // RCS "B" Quad-2 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[10], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_11: // RCS "B" oxidizer shut off valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_12: // RCS "B" Quad-4 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[12], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_13: // RCS "B" Quad-3 [ENABLE/DISABLE]
		return ActivateSwitch (event, &P2_Switch[13], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_14: // RCS "B" Ascent tank cross-feed valve [OPEN/CLOSE]
		return ActivateSwitch (event, &P2_Switch[14], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_15:
		return ActivateSwitch (event, &P2_Switch[15], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_16: // ADI Rotation Rate Scale [25 DEG/SEC, 5 DEG/SEC]
		return ActivateSwitch (event, &P2_Switch[16], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P2_17:
		return ActivateSwitch (event, &P2_Switch[17], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_DIAL_P2_00: 
		return ActivateSwitch (event, &P2_Dial[0], 4, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P2_01:
		return ActivateSwitch (event, &P2_Dial[1], 2, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P2_02:
		return ActivateSwitch (event, &P2_Dial[2], 3, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P2_03:
		return ActivateSwitch (event, &P2_Dial[3], 5, SOUND_ROTARY, OrbiterSoundID);
		
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::Switches_Panel2"

bool LM_COCKPIT::Switches_Panel3 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P3_01: // Main engine gimbal [ENABLE/DISABLE]
		return ActivateSwitch (event, &P3_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_02: // Descent Stage Command Override [ENABLE/DISABLE]
		return ActivateSwitch (event, &P3_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_03:
		return ActivateSwitch (event, &P3_Switch[2], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_04:
		return ActivateSwitch (event, &P3_Switch[3], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_05:
		return ActivateSwitch (event, &P3_Switch[4], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_06:
		return ActivateSwitch (event, &P3_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_07: // Pitch control mode [AUTO/PULSE/DIRECT]
		return ActivateSwitch (event, &P3_Switch[6], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_09:
		return ActivateSwitch (event, &P3_Switch[7], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_10: // Roll control mode [AUTO/PULSE/DIRECT]
		return ActivateSwitch (event, &P3_Switch[8], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_12:
		return ActivateSwitch (event, &P3_Switch[9], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_13: // Yaw control mode [AUTO/PULSE/DIRECT]
		return ActivateSwitch (event, &P3_Switch[10], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_14:
		return ActivateSwitch (event, &P3_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_15: // Event Timer [START,STOP,RESET]
		return ActivateSwitch (event, &P3_Switch[12], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_16: // Event Timer Mode [COUNTUP,COUNTDOWN]
		return ActivateSwitch (event, &P3_Switch[13], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_17:
		return ActivateSwitch (event, &P3_Switch[14], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_18:
		return ActivateSwitch (event, &P3_Switch[15], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_19: // Mission Clock slew (Minutes) [UP,DOWN]
		return ActivateSwitch (event, &P3_Switch[16], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_20:
		return ActivateSwitch (event, &P3_Switch[17], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_21:
		return ActivateSwitch (event, &P3_Switch[18], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_22: // Event Timer slew (Seconds) [UP,DOWN]
		return ActivateSwitch (event, &P3_Switch[19], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_23: // Track Light [OFF,ON]
		return ActivateSwitch (event, &P3_Switch[20], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_24:
		return ActivateSwitch (event, &P3_Switch[21], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P3_25: // Position Lights [OFF,STEADY,FLASHING]
		return ActivateSwitch (event, &P3_Switch[22], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_DIAL_P3_00:
		return ActivateSwitch (event, &P3_Dial[0], 6, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P3_01:
		return ActivateSwitch (event, &P3_Dial[1], 3, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P3_02: // Attitude Mode Control [OFF,HOLD,AUTO]
		return ActivateSwitch (event, &P3_Dial[2], 3, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P3_03:
		return ActivateSwitch (event, &P3_Dial[3], 7, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P3_04: // LAMP/TONE TEST
		return ActivateSwitch (event, &P3_Dial[4], 8, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P3_05: // Floodlight dimmer
		return ActivateSwitch (event, &P3_Dial[5], 4, SOUND_ROTARY, OrbiterSoundID);
		
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::Switches_Panel3"

bool LM_COCKPIT::Switches_Panel4 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P4_00:
		return ActivateSwitch (event, &P4_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P4_01: // Commander's RCS mode select [ROT,LIN]
		return ActivateSwitch (event, &P4_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P4_02: 
		return ActivateSwitch (event, &P4_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P4_03: // Pilot's RCS mode select [ROT,LIN]
		return ActivateSwitch (event, &P4_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
		
	} // End "switch (id)"	

	return false;
} // End "LM_COCKPIT::Switches_Panel4"

bool LM_COCKPIT::Switches_Panel5 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P5_00: // Mission Clock [START,STOP,RESET]
		return ActivateSwitch (event, &P5_Switch[0], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_01:
		return ActivateSwitch (event, &P5_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_02: // Mission Clock slew (Hours) [UP,DOWN]		
		return ActivateSwitch (event, &P5_Switch[2], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_03:
		return ActivateSwitch (event, &P5_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
	
	case AID_SWITCH_P5_04: // Mission Clock slew (Minutes) [UP,DOWN]
		return ActivateSwitch (event, &P5_Switch[4], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_05:
		return ActivateSwitch (event, &P5_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_06:  // Mission Clock slew (Seconds) [UP,DOWN]
		return ActivateSwitch (event, &P5_Switch[6], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P5_07:
		return ActivateSwitch (event, &P5_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
				
	} // End "switch (id)"
	
	return false;
} // End "LM_COCKPIT::Switches_Panel5"

bool LM_COCKPIT::Switches_Panel6 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P6_00:
		return ActivateSwitch (event, &P6_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_01:
		return ActivateSwitch (event, &P6_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_02:
		return ActivateSwitch (event, &P6_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_03:
		return ActivateSwitch (event, &P6_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_04:
		return ActivateSwitch (event, &P6_Switch[4], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_05:
		return ActivateSwitch (event, &P6_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_06:
		return ActivateSwitch (event, &P6_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_07:
		return ActivateSwitch (event, &P6_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_08:
		return ActivateSwitch (event, &P6_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_09:
		return ActivateSwitch (event, &P6_Switch[9], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_10:
		return ActivateSwitch (event, &P6_Switch[10], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P6_11:
		return ActivateSwitch (event, &P6_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_DIAL_P6_00: // Gyro 1 Reference Select
		return ActivateSwitch (event, &P6_Dial[0], 6, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P6_01: // Gyro 2 Reference Select
		return ActivateSwitch (event, &P6_Dial[1], 6, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P6_02:
		return ActivateSwitch (event, &P6_Dial[2], 4, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P6_03:
		return ActivateSwitch (event, &P6_Dial[3], 4, SOUND_ROTARY, OrbiterSoundID);
		
	} // End "switch (id)"

	return false;
} // End "LM_COCKPIT::Switches_Panel6"

bool LM_COCKPIT::Switches_Panel8 (int id, int event)
{
	switch (id)
	{
		case AID_SWITCH_P8_00:
		return ActivateSwitch (event, &P8_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_01:
		return ActivateSwitch (event, &P8_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_02:
		return ActivateSwitch (event, &P8_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_03:
		return ActivateSwitch (event, &P8_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_04:
		return ActivateSwitch (event, &P8_Switch[4], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_05:
		return ActivateSwitch (event, &P8_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_06:
		return ActivateSwitch (event, &P8_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_07:
		return ActivateSwitch (event, &P8_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_08:
		return ActivateSwitch (event, &P8_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_09:
		return ActivateSwitch (event, &P8_Switch[9], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_10:
		return ActivateSwitch (event, &P8_Switch[10], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_11:
		return ActivateSwitch (event, &P8_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_12:
		return ActivateSwitch (event, &P8_Switch[12], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_13:
		return ActivateSwitch (event, &P8_Switch[13], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_14:
		return ActivateSwitch (event, &P8_Switch[14], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P8_15:
		return ActivateSwitch (event, &P8_Switch[15], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_16:
		return ActivateSwitch (event, &P8_Switch[16], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_17:
		return ActivateSwitch (event, &P8_Switch[17], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_18:
		return ActivateSwitch (event, &P8_Switch[18], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_19:
		return ActivateSwitch (event, &P8_Switch[19], 3, SOUND_SWITCH, OrbiterSoundID);		
		
	case AID_SWITCH_P8_20:
		return ActivateSwitch (event, &P8_Switch[20], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P8_21:
		return ActivateSwitch (event, &P8_Switch[21], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::Switches_Panel8"

bool LM_COCKPIT::Switches_Panel12 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P12_00:
		return ActivateSwitch (event, &P12_Switch[0], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_01:
		return ActivateSwitch (event, &P12_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_02:
		return ActivateSwitch (event, &P12_Switch[2], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_03:
		return ActivateSwitch (event, &P12_Switch[3], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_04:
		return ActivateSwitch (event, &P12_Switch[4], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_05:
		return ActivateSwitch (event, &P12_Switch[5], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_06:
		return ActivateSwitch (event, &P12_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_07:
		return ActivateSwitch (event, &P12_Switch[7], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_08:
		return ActivateSwitch (event, &P12_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_09:
		return ActivateSwitch (event, &P12_Switch[9], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_10:
		return ActivateSwitch (event, &P12_Switch[10], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_11:
		return ActivateSwitch (event, &P12_Switch[11], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_12:
		return ActivateSwitch (event, &P12_Switch[12], 2, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_13:
		return ActivateSwitch (event, &P12_Switch[13], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_14:
		return ActivateSwitch (event, &P12_Switch[14], 3, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P12_15:
		return ActivateSwitch (event, &P12_Switch[15], 2, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_16:
		return ActivateSwitch (event, &P12_Switch[16], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_17:
		return ActivateSwitch (event, &P12_Switch[17], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_18:
		return ActivateSwitch (event, &P12_Switch[18], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_19:
		return ActivateSwitch (event, &P12_Switch[19], 2, SOUND_SWITCH, OrbiterSoundID);		
		
	case AID_SWITCH_P12_20:
		return ActivateSwitch (event, &P12_Switch[20], 2, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_SWITCH_P12_21:
		return ActivateSwitch (event, &P12_Switch[21], 3, SOUND_SWITCH, OrbiterSoundID);	
		
	case AID_DIAL_P12_00: // S-Band Antenna
		return ActivateSwitch (event, &P12_Dial[0], 23, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P12_01:
		return ActivateSwitch (event, &P12_Dial[1], 13, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P12_02:
		return ActivateSwitch (event, &P12_Dial[2], 3, SOUND_ROTARY, OrbiterSoundID);
		
	case AID_DIAL_P12_03:
		return ActivateSwitch (event, &P12_Dial[3], 4, SOUND_ROTARY, OrbiterSoundID);		
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::Switches_Panel12"

bool LM_COCKPIT::Switches_Panel14 (int id, int event)
{
	switch (id)
	{
	case AID_SWITCH_P14_00: 
		return ActivateSwitch (event, &P14_Switch[0], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_01: 
		return ActivateSwitch (event, &P14_Switch[1], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_02: 
		return ActivateSwitch (event, &P14_Switch[2], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_03:
		return ActivateSwitch (event, &P14_Switch[3], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_04: 
		return ActivateSwitch (event, &P14_Switch[4], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_05: 
		return ActivateSwitch (event, &P14_Switch[5], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_06: 
		return ActivateSwitch (event, &P14_Switch[6], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_07: 
		return ActivateSwitch (event, &P14_Switch[7], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_08:
		return ActivateSwitch (event, &P14_Switch[8], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_09:
		return ActivateSwitch (event, &P14_Switch[9], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_10: 
		return ActivateSwitch (event, &P14_Switch[10], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_11: 
		return ActivateSwitch (event, &P14_Switch[11], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_12: 
		return ActivateSwitch (event, &P14_Switch[12], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_13: 
		return ActivateSwitch (event, &P14_Switch[13], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_14: 
		return ActivateSwitch (event, &P14_Switch[14], 2, SOUND_SWITCH, OrbiterSoundID);
		
	case AID_SWITCH_P14_15:
		return ActivateSwitch (event, &P14_Switch[15], 2, SOUND_SWITCH, OrbiterSoundID);

	case AID_DIAL_P14_00: // S-Band Antenna
		return ActivateSwitch (event, &P14_Dial[0], 10, SOUND_ROTARY, OrbiterSoundID);
			
	} // End "switch (id)"
	return false;
} // End "LM_COCKPIT::Switches_Panel14"