// ==============================================================
//		ORBITER MODULE: 'SPIDER' LUNAR EXCURSION MODULE
//			A custom Vessel For Orbiter 2010/2011
//  
// LEM_Switches.h
// Panel Switch Definitions and Labels
// ==============================================================
#pragma once

#define SWITCH_MAN_CONT		P1_Switch[8]	// Primary Manual Control [CDR/LMP] 
#define SWITCH_GUIDECONT	P1_Switch[17]	// Guidence System reference orientation [Y+/Z+] (Orbiter Normal vs. Tailsitter)
#define SWITCH_ALTMODE		P1_Switch[18]	// Altitude/Range indicator mode select [NAV 1, ALT, NAV 2]
#define SWITCH_ADIRATESCALE	P2_Switch[16]	// ADI Rotation Rate Scale [25 DEG/SEC, 5 DEG/SEC]

#define SWITCH_TRACKLIGHT	P3_Switch[20]	// Track Light [OFF,ON]
#define SWITCH_POSLIGHT		P3_Switch[22]	// Position Lights [OFF,STEADY,FLASHING]

#define RSWITCH_LIGHTING	P3_Dial[4]		// 
#define DIAL_DIMMER			P3_Dial[5]		// Floodlight dimmer

#define SWITCH_CMD_RCSVERN	P4_Switch[0]	// Commander's RCS vernier [COARSE,FINE]
#define SWITCH_CMD_RCSMODE	P4_Switch[1]	// Commander's RCS mode select [ROT,LIN]
#define SWITCH_LMP_RCSVERN	P4_Switch[2]	// Pilot's RCS vernier [COARSE,FINE]
#define SWITCH_LMP_RCSMODE	P4_Switch[3]	// Pilot's RCS mode select [ROT,LIN]

// ==============================================================
// Guidance
// ==============================================================
#define SWITCH_CMD_ATTMON	P1_Switch[1]	// Comander's ADI mode select
#define SWITCH_CMD_RATEMON	P1_Switch[0]	// Comander's Rate/Error mode select
#define SWITCH_LMP_ATTMON	P2_Switch[17]	// Pilots's ADI mode select
#define SWITCH_LMP_RATEMON	P2_Switch[16]	// Pilots's Rate/Error mode select

#define DIAL_ATTMODE_A		P6_Dial[0]		// 
#define DIAL_ATTMODE_B		P6_Dial[1]		// 
#define DIAL_TGTMODE_A		P6_Dial[2]		//
#define DIAL_TGTMODE_B		P6_Dial[3]		//

// ==============================================================
// Thruster Management
// ==============================================================
#define SWITCH_ENG_ARM		P1_Switch[3]	// Main Engine Arm switch [DES/OFF/ASC]
#define SWITCH_THR_CONT		P1_Switch[5]	// Throttle Control [MAN/AUTO] 
#define SWITCH_ENG_GIMBL	P3_Switch[0]	// Main engine gimbal [ENABLE/DISABLE]
#define SWITCH_CMDOVRD		P3_Switch[1]	// Descent Stage Command Override [ENABLE/DISABLE]

#define SWITCH_PITCH_CONT	P3_Switch[6]	// Pitch control mode [AUTO/PULSE/DIRECT]
#define SWITCH_ROLL_CONT	P3_Switch[8]	// Roll control mode [AUTO/PULSE/DIRECT]
#define SWITCH_YAW_CONT		P3_Switch[10]	// Yaw control mode [AUTO/PULSE/DIRECT]
#define RSWITCH_ATT_MODE	P3_Dial[3]		// Attitude Mode Control [OFF,HOLD,AUTO]

#define SWITCH_RCSA_QUAD1	P2_Switch[1]	// RCS "A" Quad-1 [ENABLE/DISABLE]
#define SWITCH_RCSA_QUAD2	P2_Switch[2]	// RCS "A" Quad-2 [ENABLE/DISABLE]
#define SWITCH_RCSA_QUAD3	P2_Switch[6]	// RCS "A" Quad-3 [ENABLE/DISABLE]
#define SWITCH_RCSA_QUAD4	P2_Switch[5]	// RCS "A" Quad-4 [ENABLE/DISABLE]
#define SWITCH_RCSB_QUAD1	P2_Switch[9]	// RCS "B" Quad-1 [ENABLE/DISABLE]
#define SWITCH_RCSB_QUAD2	P2_Switch[10]	// RCS "B" Quad-2 [ENABLE/DISABLE]
#define SWITCH_RCSB_QUAD3	P2_Switch[13]	// RCS "B" Quad-3 [ENABLE/DISABLE]
#define SWITCH_RCSB_QUAD4	P2_Switch[12]	// RCS "B" Quad-4 [ENABLE/DISABLE]

// ==============================================================
// Propellant Management
// ==============================================================
#define SWITCH_PROPQTY_MON	P1_Switch[15]	// Propellant Quantity Monitoring [ASC/DES/OFF]

#define SWITCH_ASC_HeREG1	P1_Switch[11]	// Ascent Engine HeREG1 [OPEN/CLOSE]
#define SWITCH_ASC_HeREG2	P1_Switch[13]	// Ascent Engine HeREG2 [OPEN/CLOSE]

#define SWITCH_DES_HeREG1	P1_Switch[12]	// Descent Engine HeREG1 [OPEN/CLOSE]
#define SWITCH_DES_HeREG2	P1_Switch[14]	// Descent Engine HeREG2 [OPEN/CLOSE]

#define SWITCH_RCS_PROPSOVA	P2_Switch[0]	// RCS "A" propellant shut off valve [OPEN/CLOSE]
#define SWITCH_RCS_PROPSOVB	P2_Switch[8]	// RCS "B" propellant shut off valve [OPEN/CLOSE]

#define SWITCH_RCS_OXSOVA	P2_Switch[4]	// RCS "A" oxidizer shut off valve [OPEN/CLOSE]
#define SWITCH_RCS_OXSOVB	P2_Switch[11]	// RCS "B" oxidizer shut off valve [OPEN/CLOSE]

#define SWITCH_RCS_ASCXFDA	P2_Switch[7]	// RCS "A" Ascent tank cross-feed valve [OPEN/CLOSE]
#define SWITCH_RCS_ASCXFDB	P2_Switch[14]	// RCS "B" Ascent tank cross-feed valve [OPEN/CLOSE]
#define SWITCH_RCS_XFEED	P2_Switch[3]	// RCS A to B loop cross-feed valve [OPEN/CLOSE]

#define SWITCH_FUELDUMP		P8_Switch[0]	// Descent stage fuel dump valve [OPEN/CLOSE]
#define SWITCH_OXDUMP		P8_Switch[1]	// Descent stage oxidizer dump valve [OPEN/CLOSE]

// ==============================================================
// Clocks
// ==============================================================
#define SWITCH_ETIME_START	P3_Switch[12]	// Event Timer [START,STOP,RESET]
#define SWITCH_ETIME_MODE	P3_Switch[13]	// Event Timer Mode [UP,DOWN]
#define SWITCH_ETIME_MIN	P3_Switch[16]	// Event Timer slew (Minutes) [UP,DOWN]
#define SWITCH_ETIME_SEC	P3_Switch[19]	// Event Timer slew (Seconds) [UP,DOWN]

#define SWITCH_MTIME_START	P5_Switch[0]	// Mission Timer [START,STOP,RESET]
#define SWITCH_MTIME_HRS	P5_Switch[2]	// Mission Timer slew (Hours) [UP,DOWN]
#define SWITCH_MTIME_MIN	P5_Switch[4]	// Mission Timer slew (Minutes) [UP,DOWN]
#define SWITCH_MTIME_SEC	P5_Switch[6]	// Mission Timer slew (Seconds) [UP,DOWN]

// ==============================================================
// Communications and Radar
// ==============================================================
#define SWITCH_SBAND_TRACK	P12_Switch[21]	// S-Band Steerable Antenna tracking mode select [MAN,AUTO,OFF]
#define DIAL_SBAND_PITCH	P12_Dial[0]		// S-Band Steerable Antenna elevation dial
#define DIAL_SBAND_YAW		P12_Dial[1]		// S-Band Steerable Antenna azimuth dial

// ==============================================================
// Thruster Management
// ==============================================================
#define SWITCH_GEARDEPLOY	P8_Switch[5]
#define SWITCH_STAGE		P8_Switch[6]
#define SWITCH_CADMASTERARM	P8_Switch[7]
#define SWITCH_RCSPRESS		P8_Switch[8]
#define SWITCH_DESVENT		P8_Switch[9]
#define SWITCH_DESPRESS		P8_Switch[10]
#define SWITCH_HeSELECT		P8_Switch[12]
#define SWITCH_ASCPRESS		P8_Switch[13]

// ==============================================================
// Life support / ECS
// ==============================================================
#define DIAL_CABINFAN		P2_Dial[1]
#define DIAL_ECSQTYMON		P2_Dial[3]


// ==============================================================
// CWA Flags
// ==============================================================
#define CWA_0				CWA_flag[0]
#define CWA_ASC_PRESS		CWA_flag[1]
#define CWA_DES_REG			CWA_flag[2]
#define CWA_DES_QTY			CWA_flag[3]
#define CWA_4				CWA_flag[4]

#define CWA_CES_AC			CWA_flag[5]
#define CWA_CES_DC			CWA_flag[6]
#define CWA_AGS				CWA_flag[7]
#define CWA_LCC				CWA_flag[8]
#define CWA_ISS				CWA_flag[9]

#define CWA_RCS_TCA			CWA_flag[10]
#define CWA_RCS_REGA		CWA_flag[11]
#define CWA_RCS_REGB		CWA_flag[12]
#define CWA_DC_BUS			CWA_flag[13]
#define CWA_14				CWA_flag[14]

#define CWA_CABIN			CWA_flag[15]
#define CWA_SUIT			CWA_flag[16]
#define CWA_17				CWA_flag[17]
#define CWA_18				CWA_flag[18]
#define CWA_19				CWA_flag[19]

#define CWA_ASC_REG			CWA_flag[20]
#define CWA_ASC_QTY			CWA_flag[21]
#define CWA_ENG_GMBL		CWA_flag[22]
#define CWA_23				CWA_flag[23]
#define CWA_24				CWA_flag[24]

#define CWA_INVERTER		CWA_flag[25]
#define CWA_BATTERY			CWA_flag[26]
#define CWA_RNDZ_RDR		CWA_flag[27]
#define CWA_LNDG_RDR		CWA_flag[28]
#define CWA_PRE_AMP			CWA_flag[29]

#define CWA_ED_RELAY		CWA_flag[30]
#define CWA_RCS				CWA_flag[31]
#define CWA_HEATER			CWA_flag[32]
#define CWA_CW_POWER		CWA_flag[33]
#define CWA_34				CWA_flag[34]

#define CWA_ECS				CWA_flag[35]
#define CWA_O2_QTY			CWA_flag[36]
#define CWA_GLYCOL			CWA_flag[37]
#define CWA_WATER_QTY		CWA_flag[38]
#define CWA_SBD_RCVR		CWA_flag[39]