// ==============================================================
//                 Orbiter Module: LM_Cockpit
//    part of the Apollo Aplications Prject for Orbiter (AAPO)
//               copyright (c) 2012 greg hlynka
//                    all rights reserved
//
// lm_cockpit.cpp
// control module for Lunar Module virtual cockpit and displays
//
// notes:
// ==============================================================

#include "LM_Systems.h"
#include "LM_Cockpit.h"
#include "LM_DescentStage.h"

// ==============================================================
// LM Cockpit class interface
// ==============================================================

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
LM_SYSTEMS::LM_SYSTEMS (LM *vessel, LM_COCKPIT *cockpit)
{
	v = vessel;
	vc = cockpit;
	int i = 0;

	Asc_Fuel_Qty = 0;
	Asc_Fuel_Temp = 0;
	Asc_Fuel_Press = 0;
	Asc_O2_Temp = 0;
	Asc_O2_Press = 0;
	
	Des_Fuel_Qty = 0;
	Des_Fuel_Temp = 0;
	Des_Fuel_Press = 0;
	Des_O2_Temp = 0;
	Des_O2_Press = 0;
	
	Display_Fuel_Qty = 0;
	Display_Fuel_Temp = 0;
	Display_Fuel_Press = 0;
	Display_O2_Temp = 0;
	Display_O2_Press = 0;

	for (i = 0; i < 2; i++)
	{
		RCS_Qty[i] = 0;
		RCS_Temp[i] = 0;
		RCS_Press[i] = 0;
		Display_RCS_Qty[i] = 0;
		Display_RCS_Temp[i] = 0;
		Display_RCS_Press[i] = 0;
	}

	AmbientTemp = v->GetAtmTemperature ();
	AmbientPress = v->GetAtmPressure ();

	CabinTemp	= LM_CABINTEMP;
	SuitTemp	= CabinTemp;

	CabinPress	= LM_CABINPRESS;
	SuitPress	= LM_SUITPRESS;

	GlycolTemp	= 0;
	GlycolPress = 0;

	percentCO2	= 0;										
	percentH2O	= 0;										
	percentN2	= 0;										
	percentO2	= 1;										

	EventTime	= 0;
	MissionTime = oapiGetSimTime ();

	Display_Clock = 0;
	Display_Timer = 0;
	Display_Date = oapiGetSimMJD ();

	for (i = 0; i < LM_CADCOUNT; i++) CADblown[i] = true; // Initialize CAD states
	for (i = 0; i < LM_PVALVECOUNT; i++) // Initialize valve states
	{
		valve_proc[i] = 0;
		PropValve[i] = CLOSED;
	}
}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
LM_SYSTEMS::~LM_SYSTEMS ()
{

}

// ==============================================================
// Public functions
// ==============================================================

// --------------------------------------------------------------
// Initialize LM
// --------------------------------------------------------------
void LM_SYSTEMS::InitLM ()
{
	int i = 0;

} // End "LM_SYSTEMS::InitLM"

// --------------------------------------------------------------
// Read status from scenario file
// --------------------------------------------------------------
bool LM_SYSTEMS::ParseScenarioLine (char *line)
{
	int i = 0;

	if (!_strnicmp (line, "LM_ET", 5))
	{
		sscanf (line+5, "%lf", &EventTime);	
		return true;
	}		

	else if (!_strnicmp (line, "LM_MET", 6))
	{
		sscanf (line+6, "%lf", &MissionTime);
		return true;
	}		

	else if (!_strnicmp (line, "LM_CABINTEMP", 12))
	{
		sscanf (line+12, "%lf", &CabinTemp);	
		return true;
	}		

	else if (!_strnicmp (line, "LM_CABINPRESSURE", 16))
	{
		sscanf (line+16, "%lf", &CabinPress);
		return true;
	}		

	else if (!_strnicmp (line, "LM_CABINAIRCOMP", 15))
	{
		sscanf (line+15, "%lf %lf %lf %lf", &percentCO2, &percentH2O, &percentN2, &percentO2);
		return true;
	}	

	// Load Valve States
	else if (!_strnicmp (line, "LM_VALVESTATES", 9)) 
	{					  
		line += 10;
		for (i = 0; (i < LM_PVALVECOUNT) && (*line); i++) PropValve[i] = (doorstate) (*line++ - '0');
		return true;
	}

	// Load CAD states
	else if (!_strnicmp (line, "LM_EXPLSVDEVICES", 16)) 
	{					  
		line += 17;
		for (i = 0; (i < LM_CADCOUNT) && (*line); i++)
		{
			int temp = 0;
			temp = (*line++ - '0');
			CADblown[i] = (temp != 0);
		}
		return true;
	}

	else return false;
} // End "LM_SYSTEMS::LoadState"

// --------------------------------------------------------------
// Write status to scenario file
// --------------------------------------------------------------
void LM_SYSTEMS::SaveState (FILEHANDLE scn)
{
	int i = 0;
	char cbuf[256];

	// Save event timer
	sprintf (cbuf, "%lf", EventTime);						
	oapiWriteScenario_string (scn, "LM_ET", cbuf);

	// Save mission elapsed time
	sprintf (cbuf, "%lf", MissionTime);						
	oapiWriteScenario_string (scn, "LM_MET", cbuf);

	// Save cabin temprature
	sprintf (cbuf, "%lf", CabinTemp);						
	oapiWriteScenario_string (scn, "LM_CABINTEMP", cbuf);

	// Save cabin pressure
	sprintf (cbuf, "%lf", CabinPress);						
	oapiWriteScenario_string (scn, "LM_CABINPRESSURE", cbuf);

	// Save cabin atmospheric composition
	sprintf (cbuf, "%lf %lf %lf %lf", percentCO2, percentH2O, percentN2, percentO2);						
	oapiWriteScenario_string (scn, "LM_CABINAIRCOMP", cbuf);

	// Save valve states
	for (i = 0; i < LM_PVALVECOUNT; i++) cbuf[i] = PropValve[i] + '0';
	cbuf[i] = '\0';
	oapiWriteScenario_string (scn, "LM_VALVES", cbuf);

	// Save CAD states
	for (i = 0; i < LM_CADCOUNT; i++) cbuf[i] = (int) CADblown[i] + '0';
	cbuf[i] = '\0';
	oapiWriteScenario_string (scn, "LM_EXPLSVDEVICES", cbuf);

} // End "LM_SYSTEMS::SaveState"

// --------------------------------------------------------------
// Pre-Step processes
// --------------------------------------------------------------
void LM_SYSTEMS::PreStep (double simt, double simdt, double mjd)
{		
	// Get CG Offset
	if (v->AscentConfig) CoG_offset = _V( 0.0, 0.0, 0.0);
	else CoG_offset = LM_ASC_OFFSET; 

	// Get descent stage status
	Descent = v->DescentStage_attached;

	// Synch LM status variables
	v->GetGlobalPos (gPos);				// LM's position in global frame
	rBody = v->GetGravityRef ();		// Object handle of the celestial body currently being orbited. 
	v->GetRelativePos (rBody, rPos);	// Relative position of rBody
	
	// Get Ambient Conditions
	AmbientTemp = v->GetAtmTemperature ();
	AmbientPress = v->GetAtmPressure ();
	PressDiff = CabinPress - AmbientPress;

	// Number of crewmembers onboard
	nSouls = v->UMmuCrewCount;

	// Sub-system pre-step processes
	PropellantPreStep (simt, simdt);
	ThrusterPreStep (simt, simdt);
	ECSPreStep (simt, simdt);

} // End "LM_SYSTEMS::PreStep"
 
// --------------------------------------------------------------
// Post-Step processes
// --------------------------------------------------------------
void LM_SYSTEMS::PostStep (double simt, double simdt, double mjd)
{		

	// CAD post step
	//if ((CAD_MASTER_ARM == UP) && (PowerOn))
	//{
		if ((vc->SWITCH_GEARDEPLOY == UP) && (!CAD_GEAR_DEPLOY))
		{
			CAD_GEAR_DEPLOY = v->ActivateGear ();
		}
		
		if ((vc->SWITCH_STAGE == UP) && (!CAD_STAGE_JETT))
		{
			CAD_STAGE_JETT = true;
			PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);	
		}

		if ((vc->SWITCH_RCSPRESS == UP) && (!CAD_RCS_PRESS))
		{
			CAD_RCS_PRESS = true;
			PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);	
		}

		if ((vc->SWITCH_DESPRESS == UP) && (!CAD_DES_PRESS))
		{
			CAD_DES_PRESS = true;
			PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);	
		}

		if (vc->SWITCH_ASCPRESS == UP)
		{
			switch (vc->SWITCH_HeSELECT)
			{
			case UP:
				if (!CAD_ASC_PRESS1)
				{
					CAD_ASC_PRESS1 = true;
					PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);
				}
				break;

			case CENTER:
				if ((!CAD_ASC_PRESS1) || (!CAD_ASC_PRESS1))
				{
					CAD_ASC_PRESS1 = true;
					CAD_ASC_PRESS2 = true;
					PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);
				}
				break;

			case DOWN:
				if (!CAD_ASC_PRESS2)
				{
					CAD_ASC_PRESS2 = true;
					PlayVesselWave (v->OrbiterSoundID, SOUND_EXPLSVDEVICE);
				}
				break;
			} // End "switch (vc->SWITCH_HeSELECT)"
		} // "if (vc->SWITCH_ASCPRESS == UP)"

	//} //End "if ((CAD_MASTER_ARM == UP) && (PowerOn))"

	//// If Gear deployment CAD has been blown...
	//if ((CAD_GEAR_DEPLOY) && (v->GearStatus == CLOSED)) v->ActivateGear ();

	//// If Stage Jettison CAD has been blown... 
	//if ((CAD_STAGE_JETT) && (Descent)) v->StageSep ();
	

	if (vc->SWITCH_MTIME_START != CENTER) UpdateMissionClock (simt, simdt, mjd);
	if (vc->SWITCH_ETIME_START != CENTER) UpdateEventTimer (simt, simdt, mjd);

	UpdateValves (simt, simdt);

	// Sub-system post-step processes
	PropellantPostStep (simt, simdt);
	ThrusterPostStep (simt, simdt);
	ECSPostStep (simt, simdt);


} // End "LM_SYSTEMS::PostStep"

// ==============================================================
// Private Functions
// ==============================================================

// --------------------------------------------------------------
// Update Mission clock
// --------------------------------------------------------------
void LM_SYSTEMS::UpdateMissionClock (double simt, double simdt, double mjd)
{
	double delta = 0;

	if (vc->SWITCH_MTIME_START == UP) MissionTime += simdt;		// if switch is in "UP" (Start/Run) postition add simdt to MissionTime
	else if (vc->SWITCH_MTIME_START == DOWN)					// else if switch is in "DOWN" (Reset) postition...
	{
		MissionTime = 0;										// Reset Mission Timer
		vc->SWITCH_MTIME_START = CENTER;						// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_CLOCK);			// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P5_00);		// Update panel
	}

	// Slew Hours
	if (vc->SWITCH_MTIME_HRS != CENTER)
	{
		delta = 3600;											// delta = 3600 seconds, 1 hour.

		if (vc->SWITCH_MTIME_HRS == UP) MissionTime += delta;	// If switch is in "UP" position add delta to mission time...
		else MissionTime -= delta;								// else subtract

		vc->SWITCH_MTIME_HRS = CENTER;							// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_CLOCK);		// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P5_02);		// Update panel
	}
	
	// Slew Minutes
	if (vc->SWITCH_MTIME_MIN != CENTER)
	{
		delta = 60;												// delta = 60 seconds, 1 minute

		if (vc->SWITCH_MTIME_MIN == UP) MissionTime += delta;	// If switch is in "UP" position add delta...
		else MissionTime -= delta;								

		vc->SWITCH_MTIME_MIN = CENTER;							// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_CLOCK);		// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P5_04);		// Update panel
	}

	// Slew Seconds
	if (vc->SWITCH_MTIME_SEC != CENTER)	
	{
		delta = 1;												// delta = 1 second

		if (vc->SWITCH_MTIME_SEC == UP) MissionTime += delta;	// If switch is in "UP" position add delta...
		else MissionTime -= delta;								// else subtract

		vc->SWITCH_MTIME_SEC = CENTER;							// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_CLOCK);		// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P5_06);		// Update panel
	}

} // End "LM_SYSTEMS::UpdateMissionClock"

// --------------------------------------------------------------
// Update Event Timer
// --------------------------------------------------------------
void LM_SYSTEMS::UpdateEventTimer (double simt, double simdt, double mjd)
{
	double delta = 0;
	bool countdown = (vc->SWITCH_ETIME_MODE == DOWN);

	if (vc->SWITCH_ETIME_START == UP)							// If timer switch is in the "UP" (Start/Run) postition...
	{	
		if (countdown) EventTime -= simdt;						// Check if timer is in countdown mode. If so, subtract simdt from EventTime
		else EventTime += simdt;								// otherwise add it
	}
	
	if (vc->SWITCH_ETIME_START == DOWN)							// else if timer switch is in "DOWN" (Reset) postition...
	{
		EventTime = 0;											// Set Time to 0
		vc->SWITCH_ETIME_START = CENTER;						// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_TIMER);		// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P3_12);		// Update panel
	}
	
	// Slew Minutes
	if (vc->SWITCH_ETIME_MIN != CENTER)							
	{
		delta = 60;												// delta = 60 seconds

		if (vc->SWITCH_ETIME_MIN == UP) EventTime += delta;		// If switch is in "UP" position add delta...
		else EventTime -= delta;								// else subtract it

		vc->SWITCH_ETIME_MIN = CENTER;							// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_TIMER);		// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P3_16);		// Update panel
	}

	// Slew Seconds
	if (vc->SWITCH_ETIME_SEC != CENTER)							
	{
		delta = 1;												// delta = 1 second

		if (vc->SWITCH_ETIME_SEC == UP) EventTime += delta;		// If switch is in "up" position add delta to event timer
		else EventTime -= delta;								// otherwise subtract it.

		vc->SWITCH_ETIME_SEC = CENTER;							// Return switch to nuetral position
		oapiTriggerRedrawArea (0, -1, AID_LCD_TIMER);			// Update clock
		oapiTriggerRedrawArea (0, -1, AID_SWITCH_P3_16);		// Update panel
	}

	if (EventTime > 359999) EventTime = EventTime - 359999;		// If we've	run out the clock reset it.
	if (EventTime < 0) EventTime = 0;							// Disallow negative values

} // End "LM_SYSTEMS::UpdateEventTimer"

// --------------------------------------------------------------
// Update Valve states
// --------------------------------------------------------------
void LM_SYSTEMS::UpdateValves (double simt, double simdt)
{
	int i = 0;

	// Valve post step
	for (i = 0; i < LM_PVALVECOUNT; i++)
	{
		// Restrict valve state and process values to to valid range...
		if (valve_proc[i] < 0)
		{
			PropValve[i] = CLOSED;
			valve_proc[i] = 0;
		}
		
		if (valve_proc[i] > 1)
		{
			PropValve[i] = OPEN;	
			valve_proc[i] = 1;
		}

		// Update valve states
		if (PropValve[i] > OPEN)
		{
			double	delta = simdt / 0.5;	

			if (PropValve[i] == OPENING)						// If valve is opening...
			{
				if (valve_proc[i] < 1) valve_proc[i] += delta;	// If process value is less than 1 add delta to process value
				else PropValve[i] = OPEN;						// otherwise PropValve[i] = OPEN
			}
			else												// Otherwise...
			{
				if (valve_proc[i] > 0) valve_proc[i] -= delta;	// If process value is greater than 0 subtract delta from process value
				else PropValve[i] = CLOSED;						// otherwise PropValve[i] = CLOSED
			}
		}
	} // End "for (i = 0; i < LM_PVALVECOUNT; i++)"

	//// Debuggery
	//sprintf (oapiDebugString (), "Valve Status %0.0f, valve_proc %0.3f", (float)v->PropValve[0], v->valve_proc[0]);
}

// --------------------------------------------------------------
// Manage thrusters
// --------------------------------------------------------------
void LM_SYSTEMS::ThrusterPreStep (double simt, double simdt)
{	
	double thrustlvl = 0;

	// Read switch Positions
	RCSQuadArmed_A[0] = (vc->SWITCH_RCSA_QUAD1 == UP);
	RCSQuadArmed_A[1] = (vc->SWITCH_RCSA_QUAD2 == UP);
	RCSQuadArmed_A[2] = (vc->SWITCH_RCSA_QUAD3 == UP);
	RCSQuadArmed_A[3] = (vc->SWITCH_RCSA_QUAD4 == UP);

	RCSQuadArmed_B[0] = (vc->SWITCH_RCSB_QUAD1 == UP);
	RCSQuadArmed_B[1] = (vc->SWITCH_RCSB_QUAD2 == UP);
	RCSQuadArmed_B[2] = (vc->SWITCH_RCSB_QUAD3 == UP);
	RCSQuadArmed_B[3] = (vc->SWITCH_RCSB_QUAD4 == UP);

	AscentEngineArmed = (vc->SWITCH_ENG_ARM == UP);
	DescentEngineArmed = ((Descent) && (vc->SWITCH_ENG_ARM == DOWN));

	// Set main engine thrust levels
	if (Descent) // If descent stage is attached...
	{
		v->SetThrusterLevel (v->th_ascent, 0);				// Disable ascent stage engine
		thrustlvl = v->GetThrusterLevel (v->th_descent);	// Set "thrustlvl" to descent stage engine level

		// Establish min and max throttle settings for Descent engine (10 - 92.5 %)
		if ((thrustlvl > 0.0) && (v->GetThrusterLevel (v->th_descent) < 0.1)) v->SetThrusterLevel (v->th_descent, 0.1);
		else if (thrustlvl > 0.925) v->IncThrusterLevel_SingleStep (v->th_descent, -(thrustlvl - 0.925));
	}
	else		// If descent stage is not attached 
	{
		thrustlvl = v->GetThrusterLevel (v->th_ascent);		// Set "thrustlvl" to ascent stage engine level

		// The Ascent engine had no throttle capability therefore if it's level is greater than 0% set throttle to 100%
		if (thrustlvl > 0) v->SetThrusterLevel (v->th_ascent, 1); 
	}
	
	Display_Thrust  = thrustlvl;
	Display_Throttle = v->GetThrusterGroupLevel (THGROUP_HOVER);

} // End "LM_SYSTEMS::ThrusterPreStep"

void LM_SYSTEMS::ThrusterPostStep (double simt, double simdt)
{	
	// Clear existing thruster groups (if any)
	v->DelThrusterGroup (THGROUP_MAIN);
	v->DelThrusterGroup (THGROUP_HOVER);

	// Assign main engine control groups
	if (Descent) v->CreateThrusterGroup (&v->th_descent, 1, THGROUP_HOVER);		// If descent stage is attached, assign descent engine to THGROUP_HOVER
	else v->CreateThrusterGroup (&v->th_ascent, 1, THGROUP_HOVER);				// ...Otherwise, assign ascent engine to THGROUP_HOVER

	// Assign RCS control groups

	// TEMPORARY THRUST CONTROL CONTROL GROUPS
	THRUSTER_HANDLE	th_group[4];

	// RCS control groups (Rotation)
	th_group[0]	= v->th_rcs_a[4];
	th_group[1] = v->th_rcs_a[6];
	th_group[2] = v->th_rcs_b[0];
	th_group[3] = v->th_rcs_b[2];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = v->th_rcs_a[0];
	th_group[1] = v->th_rcs_a[2];
	th_group[2] = v->th_rcs_b[4];
	th_group[3] = v->th_rcs_b[6];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = v->th_rcs_a[0];
	th_group[1] = v->th_rcs_a[6];
	th_group[2] = v->th_rcs_b[2];
	th_group[3] = v->th_rcs_b[4];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT); 

	th_group[0] = v->th_rcs_a[2];
	th_group[1] = v->th_rcs_a[4];
	th_group[2] = v->th_rcs_b[0];
	th_group[3] = v->th_rcs_b[6];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = v->th_rcs_a[5];
	th_group[1] = v->th_rcs_b[1];
	//th_group[0] = v->th_rcs_a[3];
	//th_group[1] = v->th_rcs_b[7];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = v->th_rcs_a[7];
	th_group[1] = v->th_rcs_b[3];
	//th_group[0] = v->th_rcs_a[1];
	//th_group[1] = v->th_rcs_b[5];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	// RCS control groups (Translation)
	th_group[0] = v->th_rcs_a[2];
	th_group[1] = v->th_rcs_a[6];
	th_group[2] = v->th_rcs_b[0];
	th_group[3] = v->th_rcs_b[4];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = v->th_rcs_a[0];
	th_group[1] = v->th_rcs_a[4];
	th_group[2] = v->th_rcs_b[2];
	th_group[3] = v->th_rcs_b[6];
	v->CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = v->th_rcs_b[1];
	th_group[1] = v->th_rcs_b[3];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	th_group[0] = v->th_rcs_a[5];
	th_group[1] = v->th_rcs_a[7];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = v->th_rcs_a[3];
	th_group[1] = v->th_rcs_b[5];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_FORWARD);

	th_group[0] = v->th_rcs_a[1];
	th_group[1] = v->th_rcs_b[7];
	v->CreateThrusterGroup (th_group, 2, THGROUP_ATT_BACK);
	// END TEMPORARY THRUSTER GROUPS

} // End "LM_SYSTEMS::ThrusterPostStep"

bool LM_SYSTEMS::CmdOvrdOn()
{
	if (vc->SWITCH_CMDOVRD == UP) return true;
	else return false;
}

bool LM_SYSTEMS::RCSVernierOn()
{
	if ((PilotHasControls()) && (vc->SWITCH_LMP_RCSVERN == UP)) return true;
	else if ((!PilotHasControls()) && (vc->SWITCH_CMD_RCSVERN == UP)) return true;
	else return false;
}

// --------------------------------------------------------------
// Manage Propellant Resources
// --------------------------------------------------------------
void LM_SYSTEMS::PropellantPreStep (double simt, double simdt)
{	
	//// Check propellant valve control switches
	//if (Panel2Power) // First make sure we have electrical power...
	//{
		// Ascent stage pressurant valves.
		if ((vc->SWITCH_ASC_HeREG1 == UP) && (VALVE_ASC_HeREG1 != OPEN)) VALVE_ASC_HeREG1 = OPENING;
		else if ((vc->SWITCH_ASC_HeREG1 == DOWN) && (VALVE_ASC_HeREG1 != CLOSED)) VALVE_ASC_HeREG1 = CLOSING;

		if ((vc->SWITCH_ASC_HeREG2 == UP) && (VALVE_ASC_HeREG2 != OPEN)) VALVE_ASC_HeREG2 = OPENING;
		else if ((vc->SWITCH_ASC_HeREG2 == DOWN) && (VALVE_ASC_HeREG2 != CLOSED)) VALVE_ASC_HeREG2 = CLOSING;

		// Descent stage pressurant valves.
		if ((vc->SWITCH_DES_HeREG1 == UP) && (VALVE_DES_HeREG1 != OPEN)) VALVE_DES_HeREG1 = OPENING;
		else if ((vc->SWITCH_DES_HeREG1 == DOWN) && (VALVE_DES_HeREG1 != CLOSED)) VALVE_DES_HeREG1 = CLOSING;

		if ((vc->SWITCH_DES_HeREG2 == UP) && (VALVE_DES_HeREG2 != OPEN)) VALVE_DES_HeREG2 = OPENING;
		else if ((vc->SWITCH_DES_HeREG2 == DOWN) && (VALVE_DES_HeREG2 != CLOSED)) VALVE_DES_HeREG2 = CLOSING;

		// RCS Shut-off valves.
		if ((vc->SWITCH_RCS_PROPSOVA == UP) && (VALVE_RCS_PROPSOVA != OPEN)) VALVE_RCS_PROPSOVA = OPENING;
		else if ((vc->SWITCH_RCS_PROPSOVA == DOWN) && (VALVE_RCS_PROPSOVA != CLOSED)) VALVE_RCS_PROPSOVA = CLOSING;

		if ((vc->SWITCH_RCS_PROPSOVB == UP) && (VALVE_RCS_PROPSOVB != OPEN)) VALVE_RCS_PROPSOVB = OPENING;
		else if ((vc->SWITCH_RCS_PROPSOVB == DOWN) && (VALVE_RCS_PROPSOVB != CLOSED)) VALVE_RCS_PROPSOVB = CLOSING;

		if ((vc->SWITCH_RCS_OXSOVA == UP) && (VALVE_RCS_OXSOVA != OPEN)) VALVE_RCS_OXSOVA = OPENING;
		else if ((vc->SWITCH_RCS_OXSOVA == DOWN) && (VALVE_RCS_OXSOVA != CLOSED)) VALVE_RCS_OXSOVA = CLOSING;

		if ((vc->SWITCH_RCS_OXSOVB == UP) && (VALVE_RCS_OXSOVB != OPEN)) VALVE_RCS_OXSOVB = OPENING;
		else if ((vc->SWITCH_RCS_OXSOVB == DOWN) && (VALVE_RCS_OXSOVB != CLOSED)) VALVE_RCS_OXSOVB = CLOSING;

		// RCS cross-feed valves
		if ((vc->SWITCH_RCS_ASCXFDA == UP) && (VALVE_RCS_ASCXFDA != OPEN)) VALVE_RCS_ASCXFDA = OPENING;
		else if ((vc->SWITCH_RCS_ASCXFDA == DOWN) && (VALVE_RCS_ASCXFDA != CLOSED)) VALVE_RCS_ASCXFDA = CLOSING;

		if ((vc->SWITCH_RCS_ASCXFDB == UP) && (VALVE_RCS_ASCXFDB != OPEN)) VALVE_RCS_ASCXFDB = OPENING;
		else if ((vc->SWITCH_RCS_ASCXFDB == DOWN) && (VALVE_RCS_ASCXFDB != CLOSED)) VALVE_RCS_ASCXFDB = CLOSING;

		if ((vc->SWITCH_RCS_XFEED == UP) && (VALVE_RCS_XFEED != OPEN)) VALVE_RCS_XFEED = OPENING;
		else if ((vc->SWITCH_RCS_XFEED == DOWN) && (VALVE_RCS_XFEED != CLOSED)) VALVE_RCS_XFEED = CLOSING;

		// Descent stage dump valves
		if ((vc->SWITCH_FUELDUMP == UP) && (VALVE_FUELDUMP != OPEN)) VALVE_FUELDUMP = OPENING;
		else if ((vc->SWITCH_FUELDUMP == DOWN) && (VALVE_FUELDUMP != CLOSED)) VALVE_FUELDUMP = CLOSING;

		if ((vc->SWITCH_OXDUMP == UP) && (VALVE_OXDUMP != OPEN)) VALVE_OXDUMP = OPENING;
		else if ((vc->SWITCH_OXDUMP == DOWN) && (VALVE_OXDUMP != CLOSED)) VALVE_OXDUMP = CLOSING;
	//} //End Panel2Power

	// Get propellant mass	
	Asc_Fuel_Qty = v->GetPropellantMass (v->ph_ascent);
	Des_Fuel_Qty = v->GetPropellantMass (v->ph_descent);

	if (vc->SWITCH_PROPQTY_MON == UP) Display_Fuel_Qty = Asc_Fuel_Qty;
	else Display_Fuel_Qty = Des_Fuel_Qty;

	for (int i = 0; i < 2; i ++)
	{
		RCS_Qty[i] = v->GetPropellantMass (v->ph_rcs[i]);
		Display_RCS_Qty[i] = RCS_Qty[i] / v->GetPropellantMaxMass (v->ph_rcs[i]);	
	}

} // End "LM_SYSTEMS::PropellantPreStep"

void LM_SYSTEMS::PropellantPostStep (double simt, double simdt)
{	
	int i = 0;

	// Apply ascent stage propellant state
	if (AscentPropFlow()) v->SetThrusterResource (v->th_ascent, v->ph_ascent);
	else v->SetThrusterResource (v->th_ascent, NULL);

	// Apply descent stage propellant state
	if (Descent) // Sanity check, is there a descent stage to check?
	{
		if (DescentPropFlow ()) v->SetThrusterResource (v->th_descent, v->ph_descent);	// If propellant lines are pressurized assign propellant resource
		else v->SetThrusterResource (v->th_descent, NULL);									// If not, remove it.
	}

	// Apply RCS "A Loop" propellant state
	if (RCSaPropFlow ()) for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_a[i], v->ph_rcs[0]); // If RCS A loop is pressurized assign propellant resource
	else // If not...
	{
		if (AscentPropFlow () && RCSaAscentFeed()) for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_a[i], v->ph_ascent);	// Can we get propellant from the ascent engine tanks?
		else if (RCScrossfeed() && RCSbPropFlow()) for (i = 0; i < 8; i++)v-> SetThrusterResource (v->th_rcs_a[i], v->ph_rcs[1]);		// How about from RCS B?
		else for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_a[i], NULL);													// No? oh well looks like you're SOL.
	}

	// Apply RCS "B Loop" propellant state
	if (RCSbPropFlow ()) for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_b[i], v->ph_rcs[1]); // If RCS A loop is pressurized assign propellant resource
	else // If not...
	{
		if (AscentPropFlow () && RCSbAscentFeed()) for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_b[i], v->ph_ascent);	// Can we get propellant from the ascent engine tanks?
		else if (RCScrossfeed() && RCSaPropFlow()) for (i = 0; i < 8; i++)v-> SetThrusterResource (v->th_rcs_b[i], v->ph_rcs[0]);		// How about from RCS A?
		else for (i = 0; i < 8; i++) v->SetThrusterResource (v->th_rcs_b[i], NULL);													// No? oh well looks like you're SOL.
	}

} // End "LM_SYSTEMS::PropellantPostStep"

bool LM_SYSTEMS::AscentPropFlow()
{
	if ((CAD_ASC_PRESS1) || (CAD_ASC_PRESS2))
	{
		return ((VALVE_ASC_HeREG1 == OPEN) && (VALVE_ASC_HeREG2 == OPEN));
	}
	else return false;
}

bool LM_SYSTEMS::DescentPropFlow()
{
	if (CAD_DES_PRESS)
	{
		return ((VALVE_DES_HeREG1  == OPEN) && (VALVE_DES_HeREG2 == OPEN));
	}
	else return false;
}

bool LM_SYSTEMS::RCSaPropFlow()
{
	if (CAD_RCS_PRESS)
	{
		return ((VALVE_RCS_PROPSOVA == OPEN) && (VALVE_RCS_OXSOVA == OPEN));
	}
	else return false;
}

bool LM_SYSTEMS::RCSbPropFlow()
{
	if (CAD_RCS_PRESS)
	{
		return ((VALVE_RCS_PROPSOVB == OPEN) && (VALVE_RCS_OXSOVB == OPEN));
	}
	else return false;
}

bool LM_SYSTEMS::RCSaAscentFeed()
{
	if (AscentPropFlow() && (VALVE_RCS_ASCXFDB == OPEN)) return true;
	else return false;
}

bool LM_SYSTEMS::RCSbAscentFeed()
{
	if (AscentPropFlow() && (VALVE_RCS_ASCXFDB == OPEN)) return true;
	else return false;
}

bool LM_SYSTEMS::RCScrossfeed()
{
	if (VALVE_RCS_XFEED == OPEN) return true;
	else return false;
}

// --------------------------------------------------------------
// Shut down engines and autopilots
// --------------------------------------------------------------
void LM_SYSTEMS::KillThrust (void)
{
	int i;

	// Main Engines
	if (v->th_ascent) v->SetThrusterLevel (v->th_ascent, 0);
	if (v->th_descent) v->SetThrusterLevel (v->th_descent, 0);

	// RCS
	for (i = 0; i < 8; i++) v->SetThrusterLevel (v->th_rcs_a[i], 0);
	for (i = 0; i < 8; i++) v->SetThrusterLevel (v->th_rcs_b[i], 0);

	// Autopilots
	for (i = 0; i < 8; i++) if (v->GetNavmodeState (i)) v->DeactivateNavmode (i); 

} // End LM_SYSTEMS::KillThrust ()

// --------------------------------------------------------------
// Shut down engines and autopilots
// --------------------------------------------------------------
void LM_SYSTEMS::EngineStart (void)
{
	// Main Engines
	if (Descent) v->SetThrusterLevel (v->th_descent, 0.1);
	else v->SetThrusterLevel (v->th_ascent, 0.1);

} // End LM_SYSTEMS::EngineStart ()

void LM_SYSTEMS::ECSPreStep (double simt, double simdt)
{
	// Temporary
	SuitPress = LM_SUITPRESS;

	// If hatch is open match cabin conditions to ambient.
	if (v->HatchStatus != CLOSED) 
	{
		CabinTemp = AmbientTemp;
		CabinPress = AmbientPress;
	}
	else CabinTemp = LM_CABINTEMP;

	// If CabinPress is greater than SuitPress crew open their helmets
	if (CabinPress > SuitPress) SuitPress = CabinPress;
	
	// Get O2 and H2O reserves
	Asc_O2_Qty = v->GetPropellantMass (v->ph_ascent_O2);
	Asc_H2O_Qty = v->GetPropellantMass (v->ph_ascent_H2O);
	
	if (Descent)
	{
		Des_O2_Qty = v->GetPropellantMass (v->ph_descent_O2);
		Des_H2O_Qty = v->GetPropellantMass (v->ph_descent_H2O);
	}
	else
	{
		Des_O2_Qty = 0;
		Des_H2O_Qty = 0;
	}

	// Update O2 and H2O supply gauges
	if (vc->DIAL_ECSQTYMON == 0) // Monitoring ascent stage ECS tanks
	{
		Display_O2_Qty = Asc_O2_Qty / ASC_O2MASS;
		Display_H2O_Qty = Asc_H2O_Qty / ASC_WATERMASS;
	}
	else // Monitoring descent stage ECS tanks
	{
		Display_O2_Qty = Des_O2_Qty / DES_O2MASS;
		Display_H2O_Qty = Des_H2O_Qty / DES_WATERMASS;
	}


	if ((vc->DIAL_CABINFAN == 0) && (VALVE_CABINO2 != OPEN)) VALVE_CABINO2 = OPENING;
	
	// Update ECS Temprature gauges, (convert from Kelvin to Fahrenheit and adjust for range of guage)
	Display_CabinTemp = (((1.8*(CabinTemp-273))+32)-40)/60;
	Display_SuitTemp = (((1.8*(SuitTemp-273))+32)-40)/60; 

	// Update ECS Pressure gauges, (convert Pascals to PSI and adjust for range of guage)
	Display_CabinPress =  CabinPress * PSI / 10;	
	Display_SuitPress = SuitPress * PSI / 10;

	// Update CO2 partial pressure
	Display_CO2_Level = CabinPress * percentCO2 / 133.322 / 30;		

	// Update coolant gauges
	Display_GlycolTemp = ((1.8*(GlycolTemp-273))+32)/80;		// Radiator Coolant Temprature (convert from Kelvin to Fahrenheit and adjust for range of guage)	
	Display_GlycolPress = GlycolPress * PSI / 80;			// Radiator Coolant Pressure (convert Pascals to PSI and adjust for range of guage)	

}

void LM_SYSTEMS::ECSPostStep (double simt, double simdt)
{
	double O2mass;
	if (Descent) O2mass = Des_O2_Qty;
	else O2mass = Asc_O2_Qty;

		// **** Atmosphere Revitalization Section ****
		// First, get air from the suits and/or the cabin into the system.
		double nMoles = (CabinPress * LM_CABINVOLUME) / (REGNAULT * CabinTemp);			// Ideal Gas law [PV = MRT] solved for number of moles
		if (nMoles != nMoles) nMoles = 0;
		
		double molesCO2	= nMoles * percentCO2;											// Moles of carbon dioxide in cabin atmosphere
		double molesH2O	= nMoles * percentH2O;											// Moles of water in cabin atmosphere	
		double molesN2	= nMoles * percentN2;											// Moles of nitrogen in cabin atmosphere (NOTE: For simplicities sake we'll count all other trace gases as Nitrogen)
		double molesO2	= nMoles * percentO2;											// Moles of oxygen in cabin atmosphere

		//sprintf (oapiDebugString (), "Cabin Press %0.2f, Cabin Temp %0.2f, percent CO2 %0.3f, percent H2O %0.3f, percent N2 %0.3f, percent O2 %0.3f", CabinPress*PA_TO_PSI, CabinTemp, percentCO2, percentH2O, percentN2, percentO2);
		//sprintf (oapiDebugString (), "Cabin Press %0.2f, Cabin Temp %0.2f, Moles CO2 %0.3f, Moles H2O %0.3f, Moles N2 %0.3f, Moles O2 %0.3f", CabinPress*PA_TO_PSI, CabinTemp, molesCO2, molesH2O, molesN2, molesO2);
		
		// Second, remove oxygen oxygen consumed by the crew and add CO2 and H2O.
		double O2consumed = nSouls * LM_O2CONSUMPTION * simdt;	// O2 consumption
		double CO2produced = nSouls * LM_CO2PRODUCTION * simdt;	// CO2 Production

		// sprintf (oapiDebugString (), "Cabin Press %0.2f, Cabin Temp %0.2f, Souls %i, O2 Consumed %f, CO2 Produced %f", CabinPress*PA_TO_PSI, CabinTemp, nSouls, O2consumed, CO2produced);

		if ((O2consumed / O2_MOLAR_MASS) > molesO2) // Can't consume O2 that isn't there
		{	
			O2consumed = molesO2 * O2_MOLAR_MASS; 
			CO2produced = molesO2 * CO2_MOLAR_MASS;
		}

		// NOTE: The chemical equation for cellular respiration is C6H12O6 (Glucose) + O2 x6 --> CO2 x6 + H2O x6 + Energy (as ATP) 
		molesCO2 += CO2produced / CO2_MOLAR_MASS;	
		molesO2	-= O2consumed / O2_MOLAR_MASS;

		// NOTE: Chemical equation for Lithium based CO2 scrubbers is LiHO + CO2 --> Li2CO3 + H2O

		// Third, remove CO2 from the air and simulate the reaction in the LiOH can
		if (molesCO2 > 0)
		{
			double molesCO2absorbed = molesCO2 / LM_CABINCYCLE * simdt;

			if (molesCO2absorbed > molesCO2) molesCO2absorbed = molesCO2; 
			molesCO2 -= molesCO2absorbed;

			//molesLiOH += CO2absorbed;
			//double saturation = molesLiOH/27;
		}

	////	// Fourth, use the fans to move the resulting air through the suits and/or the cabin.
	////	// Fifth, use the heat exchanger to move heat from the air to the HTS if enabled (emergency ops)
	////	// Sixth, use the water separators to remove water from the air and add it to the WMS and surge tank.

			// Seventh, use the OSCPCS to add pressure if required


	////	// Eighth, use the regenerative heat exchanger to add heat to the air if required
	////	// Ninth and optionally, simulate the system behavior if a PGA failure is detected.
	////	// Tenth, simulate the LCG water movement operation.

	////	// **** Oxygen Supply and Cabin Pressure Control Section ****
	////	// Simple, move air from tanks to the cabin as required, and move air from the cabin to space as required.

		double O2flow = 0;

		if ((CabinPress < LM_CABINPRESS) && (VALVE_CABINO2 == OPEN))
		{
			O2flow = LM_O2FLOW * simdt;				// Amount of O2 to be added to cabin
			molesO2 += O2flow / O2_MOLAR_MASS;		// Add O2 to cabin
		}	

		double O2leakage = LM_O2LEAK_HP * simdt;

		double O2delta = O2leakage + O2flow;
		
		if (O2delta > O2mass) O2delta = O2mass;
		if (O2mass > 0) O2mass -= O2delta;
		else O2mass = 0;

		if (Descent) v->SetPropellantMass (v->ph_descent_O2, O2mass);
		else v->SetPropellantMass (v->ph_ascent_O2, O2mass);

		//sprintf (oapiDebugString (), "Cabin Press %0.2f, Cabin Temp %0.2f, O2 mass %0.3f, O2 loss %f, Valve State %i, valve proc %f", CabinPress*PA_TO_PSI, CabinTemp, O2mass, (O2leakage + O2flow), (int)VALVE_CABINO2, valve_proc[14]);

	////	
	////	if (VALVE_CABINDUMP != CLOSED)
	////	{

	////	}

	////	// **** Water Management Section ****
	////	// Also relatively simple, move water from tanks to the HTS / crew / etc as required.

	////	// **** Heat Transport Section ****
	////	// First, operate pumps to move glycol/water through the loops.
	////	// Second, move heat from the equipment to the glycol.
	////	// Third, move heat from the glycol to the sublimators.
	////	// Fourth, vent steam from the sublimators overboard.
	////}

		// Cabin leakage (Occurs only when cabin is pressurized)
		double AtmLeakage	= (PressDiff / LM_CABINPRESS) * (LM_O2LEAK_LP + LM_O2LEAK_CABIN);
		double AtmLoss		= AtmLeakage * simdt;
		molesCO2 -= (AtmLoss * percentCO2) / CO2_MOLAR_MASS;
		molesH2O -= (AtmLoss * percentH2O) / H2O_MOLAR_MASS;
		molesN2 -= (AtmLoss * percentN2) / N2_MOLAR_MASS;
		molesO2 -= (AtmLoss * percentO2) / O2_MOLAR_MASS;

		// O2 HP regulator leakage

		//sprintf (oapiDebugString (), "Cabin Press %0.2f, AtmLeakage %f, AtmLoss %f, O2loss %f", CabinPress*PA_TO_PSI, AtmLeakage, AtmLoss, (AtmLoss * percentO2) / O2_MOLAR_MASS );

		// Update atmospheric properties
		double MolesGas	= molesCO2 + molesH2O + molesN2 + molesO2;					// NOTE: For simplicity's sake I'm treating all non-breathable trace gases as N2 
		if (MolesGas != 0)
		{
			percentCO2		= molesCO2 / MolesGas;										// Moles of carbon dioxide in cabin atmosphere
			percentH2O		= molesH2O / MolesGas;										// Moles of water in cabin atmosphere	
			percentN2		= molesN2 / MolesGas;										// Moles of nitrogen in cabin atmosphere
			percentO2		= molesO2 / MolesGas;										// Moles of oxygen in cabin atmosphere
			CabinPress		= (MolesGas * REGNAULT * CabinTemp) / LM_CABINVOLUME;		// Ideal Gas law [PV = MRT] solved for pressure
		}
		else
		{
			percentCO2		= 0;
			percentH2O		= 0;
			percentN2		= 0;
			percentO2		= 0;
			CabinPress		= 0;
		}

	////{
	////	// O2 Consumption
	////	if (O2mass > 0)
	////	{
	////		if (Descent) v->SetPropellantMass (v->ph_descent_O2, O2mass - O2Loss);
	////		else v->SetPropellantMass (v->ph_ascent_O2, O2mass - O2Loss);
	////	}
	////	//else v->UMmuKillCrew ();

	////	// CO2 Production
	////}
	////else // Otherwise use complex model


} // End "LM_SYSTEMS::ECSPostStep"

// ==============================================================
// Switches and Booleans
// ==============================================================

bool LM_SYSTEMS::PilotHasControls()
{
	if (vc->SWITCH_MAN_CONT == DOWN) return true;
	else return false;
}

bool LM_SYSTEMS::OrientYAxis()
{
	if (vc->SWITCH_GUIDECONT == UP) return true;
	else return false;
}

bool LM_SYSTEMS::ThrottleControlIsAuto()
{
	if (vc->SWITCH_THR_CONT == UP) return true;
	else return false;
}

bool LM_SYSTEMS::GimbalEnabled()
{
	if (vc->SWITCH_ENG_GIMBL == UP) return true;
	else return false;
}

// --------------------------------------------------------------
// Thruster Control
// --------------------------------------------------------------





// Lighting
bool LM_SYSTEMS::PosLightsOn()
{
	if (vc->SWITCH_POSLIGHT != DOWN) return true;
	else return false;
}

bool LM_SYSTEMS::PosLightsFlashing()
{
	if (vc->SWITCH_POSLIGHT == UP) return true;
	else return false;
}

bool LM_SYSTEMS::TrackLightOn()
{
	if (vc->SWITCH_TRACKLIGHT == UP) return true;
	else return false;
}
