// ==============================================================
//                ORBITER MODULE: LM Descent
//                  Part of the ORBITER SDK
//               Copyright (C) 2012 Greg Hlynka
//                    All rights reserved
//
// CSM_SLAStage.cpp
// Control module for LM Descent Stage vessel class
//
// Notes:
// ==============================================================

#pragma once
#include "orbitersdk.h"
#include "AAPO_Common.h"

#include "mesh_LaunchAdapter.h"

// ==============================================================
// Vessel Constants
// ==============================================================

const double	SLA_SIZE			= 4.25;						// mean radius [m]
const VECTOR3	SLA_CS				= { 44.83, 44.83, 22.81};	// x,y,z cross sections [m^2]
const VECTOR3	SLA_PMI				= { 1.28, 1.28, 2.06};		// principal moments of inertia (mass-normalised) [m^2]
const VECTOR3	SLA_RD				= {	0.01, 0.01, 0.01};		// rotational drag coefficients

const double	SLA_MASS			= 1840;						// Launch adapter empty mass [kg] (4,050 lbs, Wikipedia)
const VECTOR3	SLA_DOCK_POS		= { 0.00, 0.00,-2.28};		// docking port location [m]
const VECTOR3	SLA_DOCK_DIR		= { 0, 0, 1};				// docking port approach direction
const VECTOR3	SLA_DOCK_ROT		= { 0, 1, 0};				// docking port alignment direction

const VECTOR3	SLA_ATTACH_BOOSTPOS	= { 0.0, 0.0,-4.25};		// aft attachment point position
const VECTOR3	SLA_ATTACH_BOOSTDIR	= { 0, 0,-1};				// aft attachment point direction
const VECTOR3	SLA_ATTACH_BOOSTROT	= { 0, 1, 0};				// aft attachment point rotation

const VECTOR3	SLA_ATTACH_SMPOS	= { 0.0, 0.0, 4.25};		// fwd attachment point position
const VECTOR3	SLA_ATTACH_SMDIR	= { 0, 0, 1};				// fwd attachment point direction
const VECTOR3	SLA_ATTACH_SMROT	= { 0, 1, 0};				// fwd attachment point rotation

// ==============================================================
// Launch Adapter vessel class Interface
// ==============================================================
class CSM_SLA: public VESSEL3 {
public:
	CSM_SLA (OBJHANDLE hVessel, int flightmodel);
	~CSM_SLA ();

	// Custom vessel functions
	void	DefineAnimations (void);			// Define mesh animations
	void	DefineSounds (void);				// Define sound effects
	double	CalcEmptyMass (void);

	// Overloaded callback functions
	void	clbkSetClassCaps (FILEHANDLE cfg);								// Set the capabilities of the vessel class
	void	clbkLoadStateEx (FILEHANDLE scn, void *status);					// Read status from scenario file
	void	clbkSaveState (FILEHANDLE scn);									// Write status to scenario file
	void	clbkPostStep (double simt, double simdt, double mjd);			// Manage Animations and Post-Step processes
	int		clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);	// Process keyboard inputs

	// Implimentation of OrbiterSound 4.0
	int		OrbiterSoundID;

	// Handles
	MESHHANDLE			mh_adapter;
	ATTACHMENTHANDLE	ah_booster, ah_payload, ah_SM;

private:
	// Vessel status variables
	enum	doorstate	{CLOSED, OPEN, CLOSING, OPENING}	SLA_Status;

	// Animations
	UINT						anim_SLA;
	double						SLA_proc;
};