// ==============================================================
//                 ORBITER MODULE: CM_Probe
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CM_Probe.cpp
// Control module for CM_Probe vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double	PROBE_SIZE	= 0.85;							// Vessel's mean radius [m]
const VECTOR3	PROBE_CS	= { 1.02, 1.02, 2.50};			// x,y,z cross sections [m^2]
const VECTOR3	PROBE_PMI	= { 2.3, 2.3, 0.8};				// Principal moments of inertia (mass-normalised) [m^2]
const VECTOR3	PROBE_RD	= { 0.025, 0.025, 0.02};		// Rotational drag coefficients
const VECTOR3	PROBE_COM	= { 0.0, 0.0,-0.3};				// Center of mass
const double	PROBE_CW[4]	= { 0.47, 1.23, 0.47, 0.47};	// Wind resistance coefficients (forward, aft, lateral, vertical)
const double	PROBE_MASS	= 100;							// Apex (parachute cover) mass [kg]

// ==============================================================
// Vessel class interface
// ==============================================================

class CM_Probe: public VESSEL3 {
public:
	CM_Probe (OBJHANDLE hVessel, int flightmodel);
	~CM_Probe ();

	void	clbkSetClassCaps (FILEHANDLE cfg);						// Set vessel class caps
	void	clbkPostStep (double simt, double simdt, double mjd);	// Post-Step processes

private:
	double	birthdate, age, lifespan;
};